@@ -27,6 +27,15 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h225ras.h,v $
+ * Revision 1.6  2011/02/22 05:04:57  shorne
+ * Enable selectively removing capabilities based on the features PDU's advertising feature. H.460.9 now advertises in ARQ when receiving call.
+ *
+ * Revision 1.5  2011/02/20 06:55:46  shorne
+ * Fixes for H.460 to allow better selection of mesasage location in PDU. Features or Generic Data. Corrected H.460.9
+ *
+ * Revision 1.4  2009/11/29 23:31:05  shorne
+ * BUG FIX : completely disable H.460 support if remote does not support it.
+ *
  * Revision 1.3  2009/02/22 02:02:05  shorne
  * Added ability to enable SCI/SCR without needing H248 support
  *
@@ -429,12 +438,15 @@ class H225_RAS : public H323Transactor
     virtual PBoolean OnReceiveRequestInProgress(const H323RasPDU &, const H225_RequestInProgress &);
     virtual PBoolean OnReceiveRequestInProgress(const H225_RequestInProgress &);
 
-    virtual PBoolean OnSendFeatureSet(unsigned, H225_FeatureSet &) const
+    virtual PBoolean OnSendFeatureSet(unsigned, H225_FeatureSet &, PBoolean) const
     { return FALSE; }
 
     virtual void OnReceiveFeatureSet(unsigned, const H225_FeatureSet &) const
     { }
 
+    virtual void DisableFeatureSet(int) const
+    { }
+
     /**Handle unknown PDU type.
       */
     virtual PBoolean OnReceiveUnknown(
