@@ -23,128 +23,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: mediafmt.h,v $
- * Revision 1.6  2008/05/23 11:19:54  willamowius
- * switch BOOL to PBoolean to be able to compile with Ptlib 2.2.x
- *
- * Revision 1.5  2008/04/25 01:16:36  shorne
- * Added ability to set the video maximum bitrate
- *
- * Revision 1.4  2008/02/06 13:19:13  shorne
- * Critical Bug Fix: Stop Assert on unhandled mediaOptions
- *
- * Revision 1.3  2007/10/19 19:53:44  shorne
- * ported latest Video updates in OpenH323 committed after h323plus initial fork thanks
- *  Robert
- *
- * Revision 1.2  2007/08/20 19:13:28  shorne
- * Added Generic Capability support. Fixed Linux compile errors
- *
- * Revision 1.1  2007/08/06 20:50:50  shorne
- * First commit of h323plus
- *
- * Revision 1.26.2.4  2007/10/03 13:10:00  rjongbloed
- * Removed duplicate OpalMediaFormat list caused by merge clash from OPAL
- *
- * Revision 1.26.2.3  2007/08/17 08:38:22  rjongbloed
- * Back ported OPAL meda options based plug ins and H.323 generic capabilties.
- *
- * Revision 1.26.2.2  2007/03/24 23:39:42  shorne
- * More H.239 work
- *
- * Revision 1.26.2.1  2006/12/23 19:08:02  shorne
- * Plugin video codecs & sundry
- *
- * Revision 1.26  2006/09/05 23:56:57  csoutheren
- * Convert media format and capability factories to use std::string
- *
- * Revision 1.25  2006/06/23 03:18:23  shorne
- * Backport FECC (H.224) from Opal
- *
- * Revision 1.24  2005/11/30 13:05:01  csoutheren
- * Changed tags for Doxygen
- *
- * Revision 1.23  2005/08/13 19:35:57  shorne
- * Fix compile error on MSVC6
- *
- * Revision 1.22  2004/07/07 08:04:54  csoutheren
- * Added video codecs to default codec list, but H.263 is only loaded if the .so/DLL is found
- *
- * Revision 1.21  2004/07/07 03:52:12  csoutheren
- * Fixed incorrect strings returned by GetFormatName on G.711 codecs
- *
- * Revision 1.20  2004/06/30 12:31:09  rjongbloed
- * Rewrite of plug in system to use single global variable for all factories to avoid all sorts
- *   of issues with startup orders and Windows DLL multiple instances.
- *
- * Revision 1.19  2004/05/23 12:49:20  rjongbloed
- * Tidied some of the OpalMediaFormat usage after abandoning some previous
- *   code due to MSVC6 compiler bug.
- *
- * Revision 1.18  2004/05/20 02:07:28  csoutheren
- * Use macro to work around MSVC internal compiler errors
- *
- * Revision 1.17  2004/05/19 07:38:22  csoutheren
- * Changed OpalMediaFormat handling to use abstract factory method functions
- *
- * Revision 1.16  2004/05/03 00:52:23  csoutheren
- * Fixed problem with OpalMediaFormat::GetMediaFormatsList
- * Added new version of OpalMediaFormat::GetMediaFormatsList that minimses copying
- *
- * Revision 1.15  2004/04/03 10:38:24  csoutheren
- * Added in initial cut at codec plugin code. Branches are for wimps :)
- *
- * Revision 1.14.2.1  2004/03/31 11:11:59  csoutheren
- * Initial public release of plugin codec code
- *
- * Revision 1.14  2004/02/26 08:19:31  csoutheren
- * Fixed threading problem with GetMediaFormatList
- *
- * Revision 1.13  2002/12/02 03:06:26  robertj
- * Fixed over zealous removal of code when NO_AUDIO_CODECS set.
- *
- * Revision 1.12  2002/09/16 01:14:15  robertj
- * Added #define so can select if #pragma interface/implementation is used on
- *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
- *
- * Revision 1.11  2002/09/03 06:19:37  robertj
- * Normalised the multi-include header prevention ifdef/define symbol.
- *
- * Revision 1.10  2002/08/05 10:03:47  robertj
- * Cosmetic changes to normalise the usage of pragma interface/implementation.
- *
- * Revision 1.9  2002/06/25 08:30:08  robertj
- * Changes to differentiate between stright G.723.1 and G.723.1 Annex A using
- *   the OLC dataType silenceSuppression field so does not send SID frames
- *   to receiver codecs that do not understand them.
- *
- * Revision 1.8  2002/03/21 02:39:15  robertj
- * Added backward compatibility define
- *
- * Revision 1.7  2002/02/11 04:15:56  robertj
- * Put G.723.1 at 6.3kbps back to old string value of "G.723.1" to improve
- *   backward compatibility. New #define is a synonym for it.
- *
- * Revision 1.6  2002/01/22 07:08:26  robertj
- * Added IllegalPayloadType enum as need marker for none set
- *   and MaxPayloadType is a legal value.
- *
- * Revision 1.5  2001/12/11 04:27:50  craigs
- * Added support for 5.3kbps G723.1
- *
- * Revision 1.4  2001/09/21 02:49:44  robertj
- * Implemented static object for all "known" media formats.
- * Added default session ID to media format description.
- *
- * Revision 1.3  2001/05/11 04:43:41  robertj
- * Added variable names for standard PCM-16 media format name.
- *
- * Revision 1.2  2001/02/09 05:16:24  robertj
- * Added #pragma interface for GNU C++.
- *
- * Revision 1.1  2001/01/25 07:27:14  robertj
- * Major changes to add more flexible OpalMediaFormat class to normalise
- *   all information about media types, especially codecs.
+ * $Id: mediafmt.h,v 1.14 2011/10/18 13:32:30 shorne Exp $
  *
  */
 
@@ -248,7 +127,7 @@ class OpalMediaOption : public PObject
     };
 
     const H245GenericInfo & GetH245Generic() const { return m_H245Generic; }
-    void SetH245Generic(const H245GenericInfo & generic) { m_H245Generic = generic; }
+    void SetH245Generic(const H245GenericInfo & gen) { m_H245Generic = gen; }
 
   protected:
     PCaselessString m_name;
@@ -259,12 +138,14 @@ class OpalMediaOption : public PObject
     H245GenericInfo m_H245Generic;
 };
 
+#if PTLIB_VER < 2110
 #ifndef __USE_STL__
 __inline istream & operator>>(istream & strm, bool& b)
 {
    int i;strm >> i;b = i; return strm;
 }
 #endif
+#endif
 
 template <typename T>
 class OpalMediaOptionValue : public OpalMediaOption
@@ -301,7 +182,7 @@ class OpalMediaOptionValue : public OpalMediaOption
       if (temp >= m_minimum && temp <= m_maximum)
         m_value = temp;
       else {
-#ifdef __USE_STL__
+#if PTLIB_VER >= 2110 || defined(__USE_STL__)
 	   strm.setstate(ios::badbit);
 #else
 	   strm.setf(ios::badbit , ios::badbit);
@@ -317,7 +198,7 @@ class OpalMediaOptionValue : public OpalMediaOption
       const OpalMediaOptionValue * otherOption = PDownCast(const OpalMediaOptionValue, &option);
       if (otherOption == NULL)
         return GreaterThan;
-      if (m_value < otherOption->m_value)
+      if (m_value < otherOption->m_value || otherOption->m_value == 0)
         return LessThan;
       if (m_value > otherOption->m_value)
         return GreaterThan;
@@ -512,12 +393,14 @@ class OpalMediaFormat : public PCaselessString
 
     void SetPayloadType(RTP_DataFrame::PayloadTypes type) { rtpPayloadType = type; }
     enum {
+      NonRTPSessionID           = 0,
       FirstSessionID            = 1,
       DefaultAudioSessionID     = 1,
       DefaultVideoSessionID     = 2,
       DefaultDataSessionID      = 3,
-      DefaultH224SessionID      = 4,
-      DefaultExtVideoSessionID  = 5,
+      DefaultH224SessionID      = 3,
+      DefaultExtVideoSessionID  = 4,
+      DefaultFileSessionID      = 5,
       LastSessionID             = 5
     };
 
@@ -748,6 +631,13 @@ class OpalMediaFormat : public PCaselessString
       OpalMediaOption * option,
       PBoolean overwrite = FALSE
     );
+
+    
+    /** 
+      * Remove all options
+      */
+    void RemoveAllOptions() 
+    { options.RemoveAll(); }
     
     /**
       * Determine if media format has the specified option.
@@ -826,9 +716,13 @@ class OpalVideoFormat : public OpalMediaFormat
 
 #define OPAL_PCM16         "PCM-16"
 #define OPAL_G711_ULAW_64K "G.711-uLaw-64k"
+#define OPAL_G711_ULAW_64K_20 "G.711-uLaw-64k-20"
 #define OPAL_G711_ALAW_64K "G.711-ALaw-64k"
+#define OPAL_G711_ALAW_64K_20 "G.711-ALaw-64k-20"
 #define OPAL_G711_ULAW_56K "G.711-uLaw-56k"
+#define OPAL_G711_ULAW_56K_20 "G.711-uLaw-56k-20"
 #define OPAL_G711_ALAW_56K "G.711-ALaw-56k"
+#define OPAL_G711_ALAW_56K_20 "G.711-ALaw-56k-20"
 #define OPAL_G728          "G.728"
 #define OPAL_G729          "G.729"
 #define OPAL_G729A         "G.729A"
@@ -843,7 +737,9 @@ class OpalVideoFormat : public OpalMediaFormat
 
 extern char OpalPCM16[];
 extern char OpalG711uLaw64k[];
+extern char OpalG711uLaw64k20[];
 extern char OpalG711ALaw64k[];
+extern char OpalG711ALaw64k20[];
 extern char OpalG728[];
 extern char OpalG729[];
 extern char OpalG729A[];
