@@ -24,6 +24,12 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: q931.h,v $
+ * Revision 1.4  2011/05/09 07:35:47  shorne
+ * updates for changes in PTLIB v2.11. The replacement of PDICTIONARY with H323Dictionary. Updates for the changes in the PNatMethod Class. Alteration in PFactory::Create, Remove redundent code and change change logging
+ *
+ * Revision 1.3  2010/01/04 05:57:22  shorne
+ * Added SetCallingSubAddressIE and SetCalledSubAddressIE support
+ *
  * Revision 1.2  2008/05/23 11:19:59  willamowius
  * switch BOOL to PBoolean to be able to compile with Ptlib 2.2.x
  *
@@ -218,6 +224,7 @@
 #pragma interface
 #endif
 
+#include "ptlib_extras.h"
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -304,7 +311,9 @@ class Q931 : public PObject
       SignalIE                = 0x34,
       ConnectedNumberIE       = 0x4c,
       CallingPartyNumberIE    = 0x6c,
+	  CallingPartySubAddressIE= 0x6d,
       CalledPartyNumberIE     = 0x70,
+	  CalledPartySubAddressIE = 0x71,
       RedirectingNumberIE     = 0x74,
       UserUserIE              = 0x7e
     };
@@ -544,6 +553,17 @@ class Q931 : public PObject
       unsigned defScreening = 0       ///<  Default value if octet3a not present
     ) const;
 
+    void SetCallingPartySubAddress(
+      const PString & number, ///<  Number string
+      unsigned plan = 1,      ///<  1 = ISDN/Telephony numbering system
+      unsigned type = 0       ///<  0 = Unknown number type
+    );
+    PBoolean GetCallingPartySubAddress(
+      PString & number,               ///<  Number string
+      unsigned * plan = NULL,         ///<  ISDN/Telephony numbering system
+      unsigned * type = NULL          ///<  Number type
+    ) const;
+
     void SetCalledPartyNumber(
       const PString & number, ///<  Number string
       unsigned plan = 1,      ///<  1 = ISDN/Telephony numbering system
@@ -555,6 +575,17 @@ class Q931 : public PObject
       unsigned * type = NULL  ///<  Number type
     ) const;
 
+    void SetCalledPartySubAddress(
+      const PString & number, ///<  Number string
+      unsigned plan = 1,      ///<  1 = ISDN/Telephony numbering system
+      unsigned type = 0       ///<  0 = Unknown number type
+    );
+    PBoolean GetCalledPartySubAddress(
+      PString & number,               ///<  Number string
+      unsigned * plan = NULL,         ///<  ISDN/Telephony numbering system
+      unsigned * type = NULL          ///<  Number type
+    ) const;
+
     void SetRedirectingNumber(
       const PString & number, ///<  Number string
       unsigned plan = 1,      ///<  1 = ISDN/Telephony numbering system
@@ -621,7 +652,7 @@ class Q931 : public PObject
     unsigned protocolDiscriminator;
     MsgTypes messageType;
 
-    PDICTIONARY(InternalInformationElements, POrdinalKey, PBYTEArray);
+    H323DICTIONARY(InternalInformationElements, POrdinalKey, PBYTEArray);
     InternalInformationElements informationElements;
 };
 
