@@ -27,6 +27,15 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h225ras.cxx,v $
+ * Revision 1.8  2011/02/22 05:04:57  shorne
+ * Enable selectively removing capabilities based on the features PDU's advertising feature. H.460.9 now advertises in ARQ when receiving call.
+ *
+ * Revision 1.7  2011/02/20 06:55:46  shorne
+ * Fixes for H.460 to allow better selection of mesasage location in PDU. Features or Generic Data. Corrected H.460.9
+ *
+ * Revision 1.6  2009/11/29 23:31:13  shorne
+ * BUG FIX : completely disable H.460 support if remote does not support it.
+ *
  * Revision 1.5  2009/02/22 02:02:37  shorne
  * Added ability to enable SCI/SCR without needing H248 support
  *
@@ -604,23 +613,21 @@ PBoolean H225_RAS::OnReceiveRequestInProgress(const H225_RequestInProgress & /*r
 template <typename PDUType>
 static void SendGenericData(const H225_RAS * ras, unsigned code, PDUType & pdu)
 {
-
  H225_FeatureSet fs;
-    if (!ras->OnSendFeatureSet(code,fs))
-		return;
+     if (ras->OnSendFeatureSet(code,fs,false)) {
+	    if (fs.HasOptionalField(H225_FeatureSet::e_supportedFeatures)) {
+	    pdu.IncludeOptionalField(PDUType::e_genericData);
 
-	if (fs.HasOptionalField(H225_FeatureSet::e_supportedFeatures)) {
-	pdu.IncludeOptionalField(PDUType::e_genericData);
+	    H225_ArrayOf_FeatureDescriptor & fsn = fs.m_supportedFeatures;
+	    H225_ArrayOf_GenericData & data = pdu.m_genericData;
 
-	H225_ArrayOf_FeatureDescriptor & fsn = fs.m_supportedFeatures;
-	H225_ArrayOf_GenericData & data = pdu.m_genericData;
-
-		for (PINDEX i=0; i < fsn.GetSize(); i++) {
-				PINDEX lastPos = data.GetSize();
-				data.SetSize(lastPos+1);
-				data[lastPos] = fsn[i];
-		}
-	}
+		    for (PINDEX i=0; i < fsn.GetSize(); i++) {
+				    PINDEX lastPos = data.GetSize();
+				    data.SetSize(lastPos+1);
+				    data[lastPos] = fsn[i];
+		    }
+	    }
+     }
 }
 
 template <typename PDUType>
@@ -628,38 +635,25 @@ static void SendFeatureSet(const H225_RAS * ras, unsigned code, PDUType & pdu)
 {
 
  H225_FeatureSet fs;
-    if (!ras->OnSendFeatureSet(code,fs))
-		return;
+   if (ras->OnSendFeatureSet(code,fs,true)) {
+        pdu.IncludeOptionalField(PDUType::e_featureSet);
+		pdu.m_featureSet = fs;
+   }
 
-     switch (code) {
-#ifdef H323_H460
-        case H460_MessageType::e_gatekeeperRequest:
-        case H460_MessageType::e_gatekeeperConfirm:
-        case H460_MessageType::e_gatekeeperReject:
-        case H460_MessageType::e_registrationRequest:
-        case H460_MessageType::e_registrationConfirm: 
-        case H460_MessageType::e_registrationReject:
-        case H460_MessageType::e_setup:					
-        case H460_MessageType::e_callProceeding:
-            pdu.IncludeOptionalField(PDUType::e_featureSet);
-			pdu.m_featureSet = fs;
-            break;
-#endif
-        default:
-		  if (fs.HasOptionalField(H225_FeatureSet::e_supportedFeatures)) {
-			pdu.IncludeOptionalField(PDUType::e_genericData);
+   if (ras->OnSendFeatureSet(code,fs,false)) {
+	  if (fs.HasOptionalField(H225_FeatureSet::e_supportedFeatures)) {
+		pdu.IncludeOptionalField(PDUType::e_genericData);
 
-			H225_ArrayOf_FeatureDescriptor & fsn = fs.m_supportedFeatures;
-		    H225_ArrayOf_GenericData & data = pdu.m_genericData;
+		H225_ArrayOf_FeatureDescriptor & fsn = fs.m_supportedFeatures;
+	    H225_ArrayOf_GenericData & data = pdu.m_genericData;
 
-			for (PINDEX i=0; i < fsn.GetSize(); i++) {
-				 PINDEX lastPos = data.GetSize();
-				 data.SetSize(lastPos+1);
-				 data[lastPos] = fsn[i];
-			}
-		  }
-		  break;
-	 }
+		for (PINDEX i=0; i < fsn.GetSize(); i++) {
+			 PINDEX lastPos = data.GetSize();
+			 data.SetSize(lastPos+1);
+			 data[lastPos] = fsn[i];
+		}
+	  }
+    }
 }
 
 
@@ -788,7 +782,10 @@ PBoolean H225_RAS::OnReceiveGatekeeperConfirm(const H323RasPDU &, const H225_Gat
   }
 
 #ifdef H323_H460
-  ReceiveFeatureSet<H225_GatekeeperConfirm>(this, H460_MessageType::e_gatekeeperConfirm, gcf);
+  if (gcf.HasOptionalField(H225_GatekeeperConfirm::e_featureSet))
+     ReceiveFeatureSet<H225_GatekeeperConfirm>(this, H460_MessageType::e_gatekeeperConfirm, gcf);
+  else
+     DisableFeatureSet(H460_MessageType::e_gatekeeperConfirm);
 #endif
 
   return OnReceiveGatekeeperConfirm(gcf);
@@ -923,7 +920,10 @@ PBoolean H225_RAS::OnReceiveRegistrationConfirm(const H323RasPDU & pdu, const H2
     return FALSE;
 
 #ifdef H323_H460
-  ReceiveFeatureSet<H225_RegistrationConfirm>(this, H460_MessageType::e_registrationConfirm, rcf);
+  if (rcf.HasOptionalField(H225_RegistrationConfirm::e_featureSet))
+     ReceiveFeatureSet<H225_RegistrationConfirm>(this, H460_MessageType::e_registrationConfirm, rcf);
+  else
+     DisableFeatureSet(H460_MessageType::e_registrationConfirm);
 #endif
 
   return OnReceiveRegistrationConfirm(rcf);
