@@ -3,7 +3,7 @@
  *
  * H.235 security PDU's
  *
- * Open H323 Library
+ * H323Plus Library
  *
  * Copyright (c) 1998-2001 Equivalence Pty. Ltd.
  *
@@ -23,125 +23,7 @@
  *
  * Contributor(s): __________________________________
  *
- * $Log: h235auth.cxx,v $
- * Revision 1.2  2008/05/23 11:21:36  willamowius
- * switch BOOL to PBoolean to be able to compile with Ptlib 2.2.x
- *
- * Revision 1.1  2007/08/06 20:51:05  shorne
- * First commit of h323plus
- *
- * Revision 1.31.2.3  2007/07/19 19:56:10  shorne
- * added missiing secure signal PDU check
- *
- * Revision 1.31.2.2  2007/05/23 07:02:38  shorne
- * Removed VS 2005 compile warning
- *
- * Revision 1.31.2.1  2006/12/23 19:08:02  shorne
- * Plugin video codecs & sundry
- *
- * Revision 1.31  2006/02/13 11:24:23  csoutheren
- * Fixed problem with H235 authenticator factory not being populated
- *
- * Revision 1.30  2006/01/26 03:25:55  shorne
- * Caller Authentication added
- *
- * Revision 1.29  2005/02/03 02:46:07  csoutheren
- * Altered authentication functions to only update state information if authentication
- *  is correct. Thanks to Michal Zygmuntowicz
- *
- * Revision 1.28  2005/01/04 08:08:45  csoutheren
- * More changes to implement the new configuration methodology, and also to
- * attack the global static problem
- *
- * Revision 1.27  2004/11/21 12:59:07  csoutheren
- * Fixed cut and paste error affecting H235 authentication
- *
- * Revision 1.26  2004/11/20 22:00:49  csoutheren
- * Added hacks for linker problem
- *
- * Revision 1.25  2004/11/12 06:04:44  csoutheren
- * Changed H235Authentiators to use PFactory
- *
- * Revision 1.24  2003/04/30 00:28:54  robertj
- * Redesigned the alternate credentials in ARQ system as old implementation
- *   was fraught with concurrency issues, most importantly it can cause false
- *   detection of replay attacks taking out an endpoint completely.
- *
- * Revision 1.23  2003/04/01 04:47:55  robertj
- * Abstracted H.225 RAS transaction processing (RIP and secondary thread) in
- *   server environment for use by H.501 peer elements.
- *
- * Revision 1.22  2003/02/25 06:48:19  robertj
- * More work on PDU transaction abstraction.
- *
- * Revision 1.21  2003/02/12 07:40:36  robertj
- * Added more logging for authentication errors in validation.
- * Fixed problem with possible negative numbers in CAT random field.
- *
- * Revision 1.20  2003/02/11 04:44:13  robertj
- * Fixed use of asymmetrical authentication schemes such as MD5.
- *
- * Revision 1.19  2003/02/03 02:55:08  robertj
- * Fixed problem with pasing through other tokens transparently.
- *
- * Revision 1.18  2003/02/02 11:46:32  robertj
- * Fixed CAT selection in GRQ authentication mechanisms.
- *
- * Revision 1.17  2003/02/01 13:31:22  robertj
- * Changes to support CAT authentication in RAS.
- *
- * Revision 1.16  2003/01/08 04:40:34  robertj
- * Added more debug tracing for H.235 authenticators.
- *
- * Revision 1.15  2002/11/11 07:04:22  robertj
- * Fixed typo in trace.
- *
- * Revision 1.14  2002/11/05 00:04:21  robertj
- * Returned code back to including trailing NULL in BMPString after
- *   PString::AsUCS2() implementation changes.
- *
- * Revision 1.13  2002/10/31 07:11:16  robertj
- * Added UTF-8/UCS-2 conversion functions to PString.
- *
- * Revision 1.12  2002/08/13 05:11:03  robertj
- * Removed redundent code.
- *
- * Revision 1.11  2002/08/05 10:03:47  robertj
- * Cosmetic changes to normalise the usage of pragma interface/implementation.
- *
- * Revision 1.10  2002/08/05 05:17:41  robertj
- * Fairly major modifications to support different authentication credentials
- *   in ARQ to the logged in ones on RRQ. For both client and server.
- * Various other H.235 authentication bugs and anomalies fixed on the way.
- *
- * Revision 1.9  2002/06/24 00:11:21  robertj
- * Clarified error message during GRQ authentication.
- *
- * Revision 1.8  2002/05/17 03:40:09  robertj
- * Fixed problems with H.235 authentication on RAS for server and client.
- *
- * Revision 1.7  2001/09/21 04:55:27  robertj
- * Removed redundant code, thanks Chih-Wei Huang
- *
- * Revision 1.6  2001/09/14 00:13:39  robertj
- * Fixed problem with some athenticators needing extra conditions to be
- *   "active", so make IsActive() virtual and add localId to H235AuthSimpleMD5
- *
- * Revision 1.5  2001/09/13 01:15:20  robertj
- * Added flag to H235Authenticator to determine if gkid and epid is to be
- *   automatically set as the crypto token remote id and local id.
- *
- * Revision 1.4  2001/08/14 04:26:46  robertj
- * Completed the Cisco compatible MD5 authentications, thanks Wolfgang Platzer.
- *
- * Revision 1.3  2001/08/13 10:03:54  robertj
- * Fixed problem if do not have a localId when doing MD5 authentication.
- *
- * Revision 1.2  2001/08/10 13:49:35  robertj
- * Fixed alpha Linux warning.
- *
- * Revision 1.1  2001/08/10 11:03:52  robertj
- * Major changes to H.235 support in RAS to support server.
+ * $Id: h235auth.cxx,v 1.12 2011/10/27 01:37:12 shorne Exp $
  *
  */
 
@@ -164,12 +46,28 @@
 
 #define new PNEW
 
-namespace PWLibStupidLinkerHacks {
-  int h235AuthLoader;
-};
-
 /////////////////////////////////////////////////////////////////////////////
 
+#ifdef H323_H235
+
+static const char H235AuthenticatorPluginBaseClass[] = "H235Authenticator";
+
+#if PTLIB_VER >= 2110
+template <> H235Authenticator * PDevicePluginFactory<H235Authenticator>::Worker::Create(const PDefaultPFactoryKey & type) const
+#else
+template <> H235Authenticator * PDevicePluginFactory<H235Authenticator>::Worker::Create(const PString & type) const
+#endif
+{
+  return H235Authenticator::CreateAuthenticator(type);
+}
+
+typedef PDevicePluginAdapter<H235Authenticator> PDevicePluginH235;
+PFACTORY_CREATE(PFactory<PDevicePluginAdapterBase>, PDevicePluginH235, H235AuthenticatorPluginBaseClass, true);
+
+#endif
+
+///////////////////////////////////////////////////////////////////////////////
+
 H235Authenticator::H235Authenticator()
 {
   enabled = TRUE;
@@ -182,6 +80,54 @@ H235Authenticator::H235Authenticator()
 }
 
 
+PStringArray H235Authenticator::GetAuthenticatorList()
+{
+    PStringArray authList;
+      PFactory<H235Authenticator>::KeyList_T keyList = PFactory<H235Authenticator>::GetKeyList();
+      PFactory<H235Authenticator>::KeyList_T::const_iterator r;
+      for (r = keyList.begin(); r != keyList.end(); ++r) {
+           authList.AppendString(*r);
+      }
+    return authList;
+}
+
+#ifdef H323_H235
+PBoolean H235Authenticator::GetAuthenticatorCapabilities(const PString & deviceName, H235Authenticator::Capabilities * caps, PPluginManager * pluginMgr)
+{
+  if (pluginMgr == NULL)
+    pluginMgr = &PPluginManager::GetPluginManager();
+  
+  return pluginMgr->GetPluginsDeviceCapabilities(H235AuthenticatorPluginBaseClass,"",deviceName, caps);
+}
+
+H235Authenticator * H235Authenticator::CreateAuthenticator(const PString & authname, PPluginManager * pluginMgr)
+{
+  if (pluginMgr == NULL)
+    pluginMgr = &PPluginManager::GetPluginManager();
+
+  return (H235Authenticator *)pluginMgr->CreatePluginsDeviceByName(authname, H235AuthenticatorPluginBaseClass,0);
+}
+
+H235Authenticator * H235Authenticator::CreateAuthenticatorByID(const PString & identifier)
+{
+  PBoolean found=false;
+  PStringArray list = GetAuthenticatorList();
+  for (PINDEX i=0; i < list.GetSize(); ++i) {
+      Capabilities caps;
+      if (GetAuthenticatorCapabilities(list[i],&caps)) {
+	      for (std::list<Capability>::const_iterator r = caps.capabilityList.begin(); r != caps.capabilityList.end(); ++r)
+              if (PString(r->m_identifier) ==  identifier) {
+                  found = true;
+                  break;
+              }
+      }
+      if (found)
+         return CreateAuthenticator(list[i]);
+  }
+  return NULL;
+}
+#endif
+
 void H235Authenticator::PrintOn(ostream & strm) const
 {
   PWaitAndSignal m(mutex);
@@ -353,7 +299,18 @@ PBoolean H235Authenticator::AddCapability(unsigned mechanism,
 
 void H235Authenticator::SetConnection(H323Connection * con)
 {
-	connection = con;
+    connection = con;
+}
+
+PBoolean H235Authenticator::GetAlgorithms(PStringList & /*algorithms*/) const
+{
+    return false;
+}
+
+PBoolean H235Authenticator::GetAlgorithmDetails(const PString & /*algorithm*/, 
+                                    PString & /*sslName*/, PString & /*description*/)
+{
+    return false;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -459,18 +416,18 @@ void H235Authenticators::PrepareSignalPDU(unsigned code,
   for (PINDEX i = 0; i < GetSize(); i++) {
     H235Authenticator & authenticator = (*this)[i];
      if (authenticator.IsSecuredSignalPDU(code, FALSE) &&
-			authenticator.PrepareTokens(clearTokens, cryptoTokens)) {
+            authenticator.PrepareTokens(clearTokens, cryptoTokens)) {
       PTRACE(4, "H235EP\tPrepared SignalPDU with authenticator " << authenticator);
-	   }
+       }
   }
 }
 
 
 H235Authenticator::ValidationResult
        H235Authenticators::ValidateSignalPDU(unsigned code,
-											const PASN_Array & clearTokens,
-											const PASN_Array & cryptoTokens,
-											const PBYTEArray & rawPDU) const
+                                            const PASN_Array & clearTokens,
+                                            const PASN_Array & cryptoTokens,
+                                            const PBYTEArray & rawPDU) const
 {
 
   H235Authenticator::ValidationResult finalresult = H235Authenticator::e_Absent;
@@ -479,61 +436,175 @@ H235Authenticator::ValidationResult
     H235Authenticator & authenticator = (*this)[i];
     if (authenticator.IsSecuredSignalPDU(code, TRUE)) {
 
-		H235Authenticator::ValidationResult result = authenticator.ValidateTokens(clearTokens, cryptoTokens, rawPDU);
-		  switch (result) {
-			case H235Authenticator::e_OK :
-			  PTRACE(4, "H235EP\tAuthenticator " << authenticator << " succeeded");
-			  finalresult = result;
-			  break;
+        H235Authenticator::ValidationResult result = authenticator.ValidateTokens(clearTokens, cryptoTokens, rawPDU);
+          switch (result) {
+            case H235Authenticator::e_OK :
+              PTRACE(4, "H235EP\tAuthenticator " << authenticator << " succeeded");
+              finalresult = result;
+              break;
 
-			case H235Authenticator::e_Absent :
-			  PTRACE(4, "H235EP\tAuthenticator " << authenticator << " absent from PDU");
-			  authenticator.Disable();
-			  break;
+            case H235Authenticator::e_Absent :
+              PTRACE(4, "H235EP\tAuthenticator " << authenticator << " absent from PDU");
+              authenticator.Disable();
+              break;
 
-			case H235Authenticator::e_Disabled :
-			  PTRACE(4, "H235EP\tAuthenticator " << authenticator << " disabled");
-			  break;
+            case H235Authenticator::e_Disabled :
+              PTRACE(4, "H235EP\tAuthenticator " << authenticator << " disabled");
+              break;
 
-			default : // Various other failure modes
-			  PTRACE(4, "H235EP\tAuthenticator " << authenticator << " failed: " << (int)result);
-			    if (finalresult != H235Authenticator::e_OK) 
-									finalresult = result;
-			  break;
-		  }	
+            default : // Various other failure modes
+              PTRACE(4, "H235EP\tAuthenticator " << authenticator << " failed: " << (int)result);
+                if (finalresult != H235Authenticator::e_OK) 
+                                    finalresult = result;
+              break;
+          }    
 
     } else {
-		authenticator.Disable();
-	}
+        authenticator.Disable();
+    }
 
   }
 
   return finalresult;
 }
 
+#ifdef H323_H235
+PStringArray GetIdentifiers(const PASN_Array & clearTokens, const PASN_Array & cryptoTokens)
+{
+
+ PStringArray ids;
+
+  for (PINDEX i = 0; i < clearTokens.GetSize(); i++) {
+    ids.AppendString(((const H235_ClearToken &)clearTokens[i]).m_tokenOID.AsString());
+  }
+
+  for (PINDEX i = 0; i < cryptoTokens.GetSize(); i++) {
+      if ((cryptoTokens.GetTag() == H225_CryptoH323Token::e_cryptoEPPwdHash) ||
+         (cryptoTokens.GetTag() == H225_CryptoH323Token::e_cryptoGKPwdHash) ||
+         (cryptoTokens.GetTag() == H225_CryptoH323Token::e_cryptoEPPwdEncr) ||
+         (cryptoTokens.GetTag() == H225_CryptoH323Token::e_cryptoGKPwdEncr) ||
+         (cryptoTokens.GetTag() == H225_CryptoH323Token::e_cryptoEPCert) ||
+         (cryptoTokens.GetTag() == H225_CryptoH323Token::e_cryptoGKCert) ||
+         (cryptoTokens.GetTag() == H225_CryptoH323Token::e_cryptoFastStart)) {
+            PTRACE(5,"H235\tReceived unsupported Token: " << cryptoTokens[i]);
+      } else if (cryptoTokens.GetTag() == H225_CryptoH323Token::e_nestedcryptoToken) {
+         const H235_CryptoToken & nestedCryptoToken = (const H235_CryptoToken &)cryptoTokens[i];
+         if (nestedCryptoToken.GetTag() != H235_CryptoToken::e_cryptoHashedToken) {
+            PTRACE(5,"H235\tReceived unsupported Nested Token: " << cryptoTokens[i]);
+            break;
+         }
+         const H235_CryptoToken_cryptoHashedToken & cryptoHashedToken = nestedCryptoToken;
+         ids.AppendString(cryptoHashedToken.m_tokenOID.AsString());    
+      }
+  }
+  return ids;
+}
+
+PBoolean H235Authenticators::CreateAuthenticators(const PASN_Array & clearTokens, const PASN_Array & cryptoTokens)
+{
+    if (clearTokens.GetSize() == 0 && cryptoTokens.GetSize() == 0)
+        return false;
+    
+    PStringArray identifiers = GetIdentifiers(clearTokens, cryptoTokens);
+    for (PINDEX i= 0; i<identifiers.GetSize(); ++i) {
+       PBoolean found = false;
+       for (PINDEX j=0; j< this->GetSize(); ++j) {
+           H235Authenticator & auth = (*this)[j];
+           if (auth.IsMatch(identifiers[i]))  {
+               found = true;
+               break; 
+           }
+       }
+       if (!found)
+         this->Append(H235Authenticator::CreateAuthenticatorByID(identifiers[i]));
+    }
+    return true;
+}
+
+PBoolean H235Authenticators::CreateAuthenticator(const PString & name)
+{
+   H235Authenticator * newAuth = H235Authenticator::CreateAuthenticator(name);
+
+   if (!newAuth) 
+     return false;
+
+   this->Append(newAuth);
+   return true;
+}
+
+PBoolean H235Authenticators::SupportsEncryption(PStringArray & list) const
+{
+   PBoolean found = false;
+   for (PINDEX j=0; j< this->GetSize(); ++j) {
+       H235Authenticator & auth = (*this)[j];
+       if (auth.GetApplication() == H235Authenticator::MediaEncryption)  {
+           list.AppendString(auth.GetName());
+           found = true;
+       }
+   }
+   return found;
+}
+
+PBoolean H235Authenticators::SupportsEncryption() const
+{
+    PStringArray list;
+    return SupportsEncryption(list);
+}
+
+PBoolean H235Authenticators::GetAlgorithms(PStringList & algorithms) const
+{
+   PBoolean found = false;
+   for (PINDEX j=0; j< this->GetSize(); ++j) {
+       H235Authenticator & auth = (*this)[j];
+       if (auth.GetApplication() == H235Authenticator::MediaEncryption)  {
+           PStringList algs;
+           if (auth.GetAlgorithms(algs)) {
+              for (PINDEX i=0; i<algs.GetSize(); ++i)
+                     algorithms.AppendString(algs[i]);
+              found = true;
+           }
+       }
+   }
+   return found; 
+}
+
+PBoolean H235Authenticators::GetAlgorithmDetails(const PString & algorithm, PString & sslName, PString & description)
+{
+   for (PINDEX j=0; j< this->GetSize(); ++j) {
+       H235Authenticator & auth = (*this)[j];
+       if (auth.GetApplication() == H235Authenticator::MediaEncryption)  {
+           if (auth.GetAlgorithmDetails(algorithm, sslName, description)) 
+              return true;
+       }
+   }
+   return false;
+}
+
+#endif
+
 ///////////////////////////////////////////////////////////////////////////////
 
 PBoolean H235AuthenticatorList::HasUserName(PString UserName) const
 {
-	for (PINDEX i = 0; i < GetSize(); i++) {
+    for (PINDEX i = 0; i < GetSize(); i++) {
         H235AuthenticatorInfo & info = (*this)[i];
-			if (UserName == info.UserName)
-				return TRUE;
-	}
+            if (UserName == info.UserName)
+                return TRUE;
+    }
     return FALSE;
 }
 
 void H235AuthenticatorList::LoadPassword(PString UserName, PString & pass) const
 {
-	for (PINDEX i = 0; i < GetSize(); i++) {
+    for (PINDEX i = 0; i < GetSize(); i++) {
         H235AuthenticatorInfo & info = (*this)[i];
-		if (UserName == info.UserName) {
-			if (info.isHashed) {
-				pass = PasswordDecrypt(info.Password);
-			} else 
-				pass = info.Password;
-		}
-	}
+        if (UserName == info.UserName) {
+            if (info.isHashed) {
+                pass = PasswordDecrypt(info.Password);
+            } else 
+                pass = info.Password;
+        }
+    }
 }
 
 void H235AuthenticatorList::Add(PString username, PString password, PBoolean isHashed)
@@ -549,50 +620,54 @@ void H235AuthenticatorList::Add(PString username, PString password, PBoolean isH
 PString H235AuthenticatorList::PasswordEncrypt(const PString &clear) const
 {
 
-int keyFilled = 0;
+    int keyFilled = 0;
 
-const PString key = AuthenticatorListHashKey;
-	
-	PTEACypher::Key thekey;
-	memset(&thekey, keyFilled, sizeof(PTEACypher::Key));
-	memcpy(&thekey, const_cast<PString &>(key).GetPointer(), min(sizeof(PTEACypher::Key), size_t(key.GetLength())));
-	PTEACypher cypher(thekey);
-	return cypher.Encode(clear);	
+    const PString key = AuthenticatorListHashKey;
+    
+    PTEACypher::Key thekey;
+    memset(&thekey, keyFilled, sizeof(PTEACypher::Key));
+    memcpy(&thekey, (const char *)key, min(sizeof(PTEACypher::Key), size_t(key.GetLength())));
+    PTEACypher cypher(thekey);
+    return cypher.Encode(clear);    
 }
 
 PString H235AuthenticatorList::PasswordDecrypt(const PString &encrypt) const
 {
 
-int keyFilled = 0;
+    int keyFilled = 0;
 
-const PString key = AuthenticatorListHashKey;
+    const PString key = AuthenticatorListHashKey;
 
-	PTEACypher::Key thekey;
-	memset(&thekey, keyFilled, sizeof(PTEACypher::Key));
-	memcpy(&thekey, const_cast<PString &>(key).GetPointer(), min(sizeof(PTEACypher::Key), size_t(key.GetLength())));
-	PTEACypher cypher(thekey);
-	return cypher.Decode(encrypt);	
+    PTEACypher::Key thekey;
+    memset(&thekey, keyFilled, sizeof(PTEACypher::Key));
+    memcpy(&thekey, (const char *)key, min(sizeof(PTEACypher::Key), size_t(key.GetLength())));
+    PTEACypher cypher(thekey);
+    return cypher.Decode(encrypt);    
 }
 
 ///////////////////////////////////////////////////////////////////////////////
 H235AuthenticatorInfo::H235AuthenticatorInfo(PString username,PString password,PBoolean ishashed)
-	: UserName(username), Password(password), isHashed(ishashed)
+    : UserName(username), Password(password), isHashed(ishashed)
 {
 }
 
 H235AuthenticatorInfo::H235AuthenticatorInfo(PSSLCertificate * cert)
-	: Certificate(cert)
+    : Certificate(cert)
 {
 }
 ///////////////////////////////////////////////////////////////////////////////
 
+#ifdef H323_H235
+H235SECURITY(MD5);
+#else
 static PFactory<H235Authenticator>::Worker<H235AuthSimpleMD5> factoryH235AuthSimpleMD5("SimpleMD5");
+#endif
 
 static const char OID_MD5[] = "1.2.840.113549.2.5";
 
 H235AuthSimpleMD5::H235AuthSimpleMD5()
 {
-	usage = AnyApplication; // Can be used either for GKAdmission or EPAuthenticstion
+    usage = AnyApplication; // Can be used either for GKAdmission or EPAuthenticstion
 }
 
 
@@ -607,6 +682,28 @@ const char * H235AuthSimpleMD5::GetName() const
   return "MD5";
 }
 
+PStringArray H235AuthSimpleMD5::GetAuthenticatorNames()
+{
+    return PStringArray("MD5");
+}
+
+#ifdef H323_H235
+PBoolean H235AuthSimpleMD5::GetAuthenticationCapabilities(H235Authenticator::Capabilities * ids)
+{
+      H235Authenticator::Capability cap;
+        cap.m_identifier = OID_MD5;
+        cap.m_cipher     = "MD5";
+        cap.m_description= "md5";
+       ids->capabilityList.push_back(cap);
+    
+    return true;
+}
+#endif
+
+PBoolean H235AuthSimpleMD5::IsMatch(const PString & identifier) const 
+{ 
+    return (identifier == PString(OID_MD5)); 
+}
 
 static PWCharArray GetUCS2plusNULL(const PString & str)
 {
@@ -635,10 +732,19 @@ H225_CryptoH323Token * H235AuthSimpleMD5::CreateCryptoToken()
   clearToken.m_tokenOID = "0.0";
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_generalID);
+    // use SetValueRaw() starting with PTLib 2.7.x to preserve the trailing null byte
+#if PTLIB_VER < 270
   clearToken.m_generalID = GetUCS2plusNULL(localId);
+#else
+  clearToken.m_generalID.SetValueRaw(GetUCS2plusNULL(localId));
+#endif
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_password);
+#if PTLIB_VER < 270
   clearToken.m_password = GetUCS2plusNULL(password);
+#else
+  clearToken.m_password.SetValueRaw(GetUCS2plusNULL(password));
+#endif
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_timeStamp);
   clearToken.m_timeStamp = (int)time(NULL);
@@ -688,11 +794,11 @@ H235Authenticator::ValidationResult H235AuthSimpleMD5::ValidateCryptoToken(
 #ifndef DISABLE_CALLAUTH
   // If has connection then EP Authenticator so CallBack to Check SenderID and Set Password
   if (connection != NULL) { 
-	 if (!connection->OnCallAuthentication(alias,password)) {
-		PTRACE(1, "H235EP\tH235AuthSimpleMD5 Authentication Fail UserName \"" << alias 
-			   << "\", not Authorised. \"");
-		return e_BadPassword;
-	 }
+     if (!connection->OnCallAuthentication(alias,password)) {
+        PTRACE(1, "H235EP\tH235AuthSimpleMD5 Authentication Fail UserName \"" << alias 
+               << "\", not Authorised. \"");
+        return e_BadPassword;
+     }
   } else {
 #endif
      if (!remoteId && alias != remoteId) {
@@ -709,10 +815,19 @@ H235Authenticator::ValidationResult H235AuthSimpleMD5::ValidateCryptoToken(
   clearToken.m_tokenOID = "0.0";
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_generalID);
+  // use SetValueRaw() starting with PTLib 2.7.x to preserve the trailing null byte
+#if PTLIB_VER < 270
   clearToken.m_generalID = GetUCS2plusNULL(alias);
+#else
+  clearToken.m_generalID.SetValueRaw(GetUCS2plusNULL(alias));
+#endif
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_password);
+#if PTLIB_VER < 270
   clearToken.m_password = GetUCS2plusNULL(password);
+#else
+  clearToken.m_password.SetValueRaw(GetUCS2plusNULL(password));
+#endif
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_timeStamp);
   clearToken.m_timeStamp = cryptoEPPwdHash.m_timeStamp;
@@ -788,7 +903,7 @@ static const char OID_CAT[] = "1.2.840.113548.10.1.2.1";
 
 H235AuthCAT::H235AuthCAT()
 {
-	usage = GKAdmission;		 /// To be used for GKAdmission
+    usage = GKAdmission;         /// To be used for GKAdmission
 }
 
 
@@ -820,7 +935,12 @@ H235_ClearToken * H235AuthCAT::CreateClearToken()
   clearToken->m_tokenOID = OID_CAT;
 
   clearToken->IncludeOptionalField(H235_ClearToken::e_generalID);
+  // use SetValueRaw() starting with PTLib 2.7.x to preserve the trailing null byte
+#if PTLIB_VER < 270
   clearToken->m_generalID = GetUCS2plusNULL(localId);
+#else
+  clearToken->m_generalID.SetValueRaw(GetUCS2plusNULL(localId));
+#endif
 
   clearToken->IncludeOptionalField(H235_ClearToken::e_timeStamp);
   clearToken->m_timeStamp = (int)time(NULL);
@@ -959,5 +1079,9 @@ PBoolean H235AuthCAT::IsSecuredPDU(unsigned rasPDU, PBoolean received) const
   }
 }
 
+// need to load into factory here otherwise doesn't load...
+#if P_SSL && !defined(H323_H235) 
+static PFactory<H235Authenticator>::Worker<H2351_Authenticator> factoryH2351_Authenticator("H2351_Authenticator");
+#endif
 
 /////////////////////////////////////////////////////////////////////////////
