@@ -19,6 +19,12 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h281.cxx,v $
+ * Revision 1.4  2011/02/20 07:03:00  shorne
+ * Improving H.224 stability by ensuring correct initialization and removing nested mutexes.
+ *
+ * Revision 1.3  2011/01/12 12:51:52  shorne
+ * H.224 bi-directional support added
+ *
  * Revision 1.2  2008/05/23 11:21:52  willamowius
  * switch BOOL to PBoolean to be able to compile with Ptlib 2.2.x
  *
@@ -468,6 +474,7 @@ OpalH281Handler::OpalH281Handler(OpalH224Handler & theH224Handler)
   remoteHasH281 = FALSE;
   localNumberOfPresets = 0;
   remoteNumberOfPresets = 0;
+  shutDown = false;
 	
   // set correct video source numbers
   for(BYTE srcnum = 0; srcnum < 6; srcnum++) {	  
@@ -498,8 +505,9 @@ OpalH281Handler::OpalH281Handler(OpalH224Handler & theH224Handler)
 
 OpalH281Handler::~OpalH281Handler()
 {
-  PWaitAndSignal m(h224Handler.GetTransmitMutex());
-	
+ // PWaitAndSignal m(h224Handler.GetTransmitMutex());
+
+  shutDown = true;
   transmitTimer.Stop();
   receiveTimer.Stop();
 }
@@ -519,7 +527,7 @@ void OpalH281Handler::StartAction(H281_Frame::PanDirection panDirection,
                                   H281_Frame::ZoomDirection zoomDirection,
                                   H281_Frame::FocusDirection focusDirection)
 {
-  PWaitAndSignal m(h224Handler.GetTransmitMutex());
+  //PWaitAndSignal m(h224Handler.GetTransmitMutex());
 
   if(transmitFrame.GetRequestType() != H281_Frame::IllegalRequest) {
 
@@ -550,7 +558,7 @@ void OpalH281Handler::StartAction(H281_Frame::PanDirection panDirection,
 
 void OpalH281Handler::StopAction()
 {
-  PWaitAndSignal m(h224Handler.GetTransmitMutex());
+  //PWaitAndSignal m(h224Handler.GetTransmitMutex());
 	
   transmitFrame.SetRequestType(H281_Frame::StopAction);
 	
@@ -562,7 +570,7 @@ void OpalH281Handler::StopAction()
 
 void OpalH281Handler::SelectVideoSource(BYTE videoSourceNumber, H281_Frame::VideoMode videoMode)
 {
-  PWaitAndSignal m(h224Handler.GetTransmitMutex());
+  //PWaitAndSignal m(h224Handler.GetTransmitMutex());
 	
   if(transmitFrame.GetRequestType() != H281_Frame::IllegalRequest) {
     StopAction();
@@ -579,7 +587,7 @@ void OpalH281Handler::SelectVideoSource(BYTE videoSourceNumber, H281_Frame::Vide
 
 void OpalH281Handler::StoreAsPreset(BYTE presetNumber)
 {
-  PWaitAndSignal m(h224Handler.GetTransmitMutex());
+  //PWaitAndSignal m(h224Handler.GetTransmitMutex());
 	
   if(transmitFrame.GetRequestType() != H281_Frame::IllegalRequest) {
     StopAction();
@@ -595,7 +603,7 @@ void OpalH281Handler::StoreAsPreset(BYTE presetNumber)
 
 void OpalH281Handler::ActivatePreset(BYTE presetNumber)
 {
-  PWaitAndSignal m(h224Handler.GetTransmitMutex());
+  //PWaitAndSignal m(h224Handler.GetTransmitMutex());
 	
   if(transmitFrame.GetRequestType() != H281_Frame::IllegalRequest) {
     StopAction();
@@ -781,7 +789,10 @@ void OpalH281Handler::OnActivatePreset(BYTE /*presetNumber*/)
 
 void OpalH281Handler::ContinueAction(PTimer &, INT)
 {
-  PWaitAndSignal(h224Handler.GetTransmitMutex());
+  //PWaitAndSignal(h224Handler.GetTransmitMutex());
+
+  if (shutDown)
+      return;
 	
   transmitFrame.SetRequestType(H281_Frame::ContinueAction);
 	
