@@ -34,6 +34,21 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323filetransfer.cxx,v $
+ * Revision 1.10  2011/10/18 13:32:30  shorne
+ * Rearranged Default Session ids so FECC is Session 3 for better interoperability.
+ *
+ * Revision 1.9  2011/04/08 15:54:47  willamowius
+ * avoid using PString::GetPointer()
+ *
+ * Revision 1.8  2011/01/10 10:31:16  shorne
+ * fixed bug wth maxBitRate values in the TCS
+ *
+ * Revision 1.7  2010/01/18 21:24:06  shorne
+ * Fixes to send complete filie paths & filenames with '0' in the name
+ *
+ * Revision 1.6  2009/10/21 10:04:02  shorne
+ * Update OID value to bring into line with OCS
+ *
  * Revision 1.5  2009/02/21 14:16:32  shorne
  * Major overhaul of the FileTransfer code
  *
@@ -67,7 +82,8 @@
 #include <h323pdu.h>
 
 
-static const char * FileTransferOID = "1.3.6.1.4.1.17090.3.1";
+static const char * FileTransferOID = "1.3.6.1.4.1.17090.1.2";
+static const char * FileTransferListOID = "1.3.6.1.4.1.17090.1.2.1";
 
 static struct  {
    int blocksize;
@@ -120,14 +136,14 @@ PBoolean H323FileTransferCapability::OnReceivedPDU(const H245_DataApplicationCap
   if (pdu.m_application.GetTag() != H245_DataApplicationCapability_application::e_genericDataCapability)
         return FALSE;
 
-  maxBitRate = pdu.m_maxBitRate;
+  //maxBitRate = pdu.m_maxBitRate*100;
   const H245_GenericCapability & genCapability = (const H245_GenericCapability &)pdu.m_application;
   return OnReceivedPDU(genCapability);
 }
 
 PBoolean H323FileTransferCapability::OnSendingPDU(H245_DataApplicationCapability & pdu) const
 {
-  pdu.m_maxBitRate = maxBitRate;
+  pdu.m_maxBitRate = maxBitRate/100;
   pdu.m_application.SetTag(H245_DataApplicationCapability_application::e_genericDataCapability);
 	
   H245_GenericCapability & genCapability = (H245_GenericCapability &)pdu.m_application;
@@ -167,7 +183,7 @@ PBoolean H323FileTransferCapability::OnReceivedPDU(const H245_GenericCapability
 
    if (pdu.HasOptionalField(H245_GenericCapability::e_maxBitRate)) {
       const PASN_Integer & bitRate = pdu.m_maxBitRate;
-      maxBitRate = bitRate;
+      maxBitRate = bitRate*100;
    }
 
    if (!pdu.HasOptionalField(H245_GenericCapability::e_collapsing)) 
@@ -208,7 +224,7 @@ PBoolean H323FileTransferCapability::OnSendingPDU(H245_GenericCapability & pdu)
 
    pdu.IncludeOptionalField(H245_GenericCapability::e_maxBitRate);
    PASN_Integer & bitRate = pdu.m_maxBitRate;
-   bitRate = maxBitRate;
+   bitRate = maxBitRate/100;
 
    // Add the Block size parameter
    H245_GenericParameter * blockparam = new H245_GenericParameter;
@@ -238,6 +254,11 @@ void H323FileTransferCapability::SetFileTransferList(const H323FileTransferList
 	m_filelist.SetMaster(true);
 }
 
+unsigned H323FileTransferCapability::GetDefaultSessionID() const
+{
+    return OpalMediaFormat::DefaultFileSessionID;
+}
+
 unsigned H323FileTransferCapability::GetSubType() const
 {
 	return H245_DataApplicationCapability_application::e_genericDataCapability;
@@ -616,7 +637,7 @@ PBoolean H323FileTransferChannel::RetreiveFileInfo(const H245_GenericInformation
 		  return FALSE;
 
 	  const PASN_ObjectId &object_id = info.m_messageIdentifier;
-      if (object_id != FileTransferOID)   // Indicates File Information
+      if (object_id != FileTransferListOID)   // Indicates File Information
 		   return FALSE;
  
 	  if (!info.HasOptionalField(H245_GenericInformation::e_messageContent))
@@ -711,7 +732,7 @@ void H323FileTransferChannel::SetFileList(H245_OpenLogicalChannel & open, H323Fi
 	  H245_GenericInformation * gcap = new H245_GenericInformation();
 	  gcap->m_messageIdentifier = *(new H245_CapabilityIdentifier(H245_CapabilityIdentifier::e_standard));
 	  PASN_ObjectId &object_id = gcap->m_messageIdentifier;
-      object_id = FileTransferOID;   // Indicates File Information
+      object_id = FileTransferListOID;   // Indicates File Information
 
   	  i++;
 	  gcap->IncludeOptionalField(H245_GenericInformation::e_subMessageIdentifier);
@@ -1005,7 +1026,7 @@ PBoolean H323FileTransferHandler::TransmitFrame(H323FilePacket & buffer, PBoolea
   transmitFrame.SetPayloadSize(buffer.GetSize());
   memmove(transmitFrame.GetPayloadPtr(),buffer.GetPointer(), buffer.GetSize());
 	
-  return session->WriteData(transmitFrame);
+  return (session && session->WriteData(transmitFrame));
 }
 
 PBoolean H323FileTransferHandler::ReceiveFrame(H323FilePacket & buffer, PBoolean & final)
@@ -1374,7 +1395,9 @@ void H323FileTransferHandler::Receive(PThread &, INT)
 				   int size = packet.GetFileSize();
 				   if (size > 0) {
                         curFileSize = size;
-					    p = filelist.GetSaveDirectory() + PDIR_SEPARATOR + curFileName;
+						PStringList saveFile;
+						saveFile = curFileName.Tokenise(PDIR_SEPARATOR);
+					    p = filelist.GetSaveDirectory() + PDIR_SEPARATOR + saveFile[saveFile.GetSize()-1];
 						delete curFile;
 						curFile = new H323FileIOChannel(p,false);
 						if (curFile->IsError(ioerr)) {
@@ -1482,7 +1505,7 @@ static PString opStr[] = {
 void H323FilePacket::attach(PString & data)
 {
 	SetSize(data.GetSize());
-	memcpy(theArray, data.GetPointer(), data.GetSize());
+	memcpy(theArray, (const char *)data, data.GetSize());
 }
 
 void H323FilePacket::BuildPROB()
@@ -1493,7 +1516,9 @@ void H323FilePacket::BuildPROB()
 
 void H323FilePacket::BuildRequest(opcodes code, const PString & filename, int filesize, int blocksize)
 {
-   PString header = opStr[code] + filename + "0octet0blksize0" + PString(blocksize) 
+   PString fn = filename;
+   fn.Replace("0","*",true);
+   PString header = opStr[code] + fn + "0octet0blksize0" + PString(blocksize) 
 	                    + "0tsize0" + PString(filesize) + "0";
    attach(header);
 
@@ -1510,7 +1535,7 @@ void H323FilePacket::BuildData(int blockid, int size)
    PString data = opStr[e_DATA] + blkstr;
 
    SetSize(size+4);
-   memcpy(theArray, data.GetPointer(), data.GetSize()); 
+   memcpy(theArray, (const char *)data, data.GetSize()); 
 }
 
 void H323FilePacket::BuildACK(int blockid, int filesize)
@@ -1550,6 +1575,7 @@ PString H323FilePacket::GetFileName() const
 
   PStringArray ar = (data.Mid(2)).Tokenise('0');
 
+  ar[0].Replace("*","0",true);
   return ar[0];
 }
 
