@@ -26,244 +26,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: h323neg.cxx,v $
- * Revision 1.2  2008/05/23 11:22:03  willamowius
- * switch BOOL to PBoolean to be able to compile with Ptlib 2.2.x
- *
- * Revision 1.1  2007/08/06 20:51:07  shorne
- * First commit of h323plus
- *
- * Revision 1.70.4.1  2007/03/24 23:39:43  shorne
- * More H.239 work
- *
- * Revision 1.70  2005/05/03 12:23:05  csoutheren
- * Unlock connection list when creating connection
- * Remove chance of race condition with H.245 negotiation timer
- * Unregister OpalPluginMediaFormat when instance is destroyed
- * Thank to Paul Cadach
- *
- * Revision 1.69  2003/01/30 01:38:19  robertj
- * Fixed some trace log errors
- *
- * Revision 1.68  2002/11/05 05:33:56  robertj
- * Fixed problem where if capability is rejected by OnReceivedCapabilitySet()
- *   it still thinks it has received them.
- *
- * Revision 1.67  2002/08/05 10:03:47  robertj
- * Cosmetic changes to normalise the usage of pragma interface/implementation.
- *
- * Revision 1.66  2002/06/27 07:11:55  robertj
- * Fixed GNU warning
- *
- * Revision 1.65  2002/06/26 08:51:19  robertj
- * Fixed deadlock if logical channel is closed via H.245 at exactly same
- *   time as being closed locally due to a channel I/O error.
- *
- * Revision 1.64  2002/05/03 03:08:40  robertj
- * Added replacementFor field in OLC when resolving conflicting channels.
- *
- * Revision 1.63  2002/01/10 04:38:29  robertj
- * Fixed problem with having incorrect constraint if getting array from caller on
- *   setting up a requestMode.
- *
- * Revision 1.62  2002/01/10 04:02:08  robertj
- * Fixed correct filling of modeRequest structure.
- *
- * Revision 1.61  2002/01/09 00:21:40  robertj
- * Changes to support outgoing H.245 RequstModeChange.
- *
- * Revision 1.60  2002/01/08 01:26:32  robertj
- * Removed PTRACE in FindChannelsBySession as it can get called a LOT.
- *
- * Revision 1.59  2002/01/01 23:24:04  craigs
- * Removed reference to myPTRACE
- *
- * Revision 1.58  2002/01/01 23:20:22  craigs
- * Added HandleAck and StartRequest implementations for T.38
- * thanks to Vyacheslav Frolov
- *
- * Revision 1.57  2001/12/22 03:10:17  robertj
- * Changed OnRequstModeChange to return ack, then actually do the change.
- *
- * Revision 1.56  2001/12/22 01:46:33  robertj
- * Fixed bug in RequestMode negotiation, failure to set sequence number.
- *
- * Revision 1.55  2001/12/14 08:36:36  robertj
- * More implementation of T.38, thanks Adam Lazur
- *
- * Revision 1.54  2001/11/11 23:07:52  robertj
- * Some clean ups after T.38 commit, thanks Adam Lazur
- *
- * Revision 1.53  2001/11/09 05:39:54  craigs
- * Added initial T.38 support thanks to Adam Lazur
- *
- * Revision 1.52  2001/09/13 04:18:57  robertj
- * Added support for "reopen" function when closing logical channel.
- *
- * Revision 1.51  2001/08/06 03:08:56  robertj
- * Fission of h323.h to h323ep.h & h323con.h, h323.h now just includes files.
- *
- * Revision 1.50  2001/05/30 23:34:54  robertj
- * Added functions to send TCS=0 for transmitter side pause.
- *
- * Revision 1.49  2001/05/09 02:34:37  robertj
- * Fixed problem with being able to start two MSD's at the same time.
- * Fixed possible race condition in code to prevent two TCS's happening
- *   at the same time.
- *
- * Revision 1.48  2001/04/26 02:01:53  robertj
- * Added workaround for more strange Cisco behaviour, stops sending RTP
- *   data if you try and do a H.245 RoundTripDelay request!!
- *
- * Revision 1.47  2001/03/16 07:11:39  robertj
- * Added logical channel open function version without locking.
- *
- * Revision 1.46  2001/03/14 03:20:27  robertj
- * Fixed possible nested mutex deadlock in logical channel negotiator.
- *
- * Revision 1.45  2001/03/06 04:44:47  robertj
- * Fixed problem where could send capability set twice. This should not be
- *   a problem except when talking to another broken stack, eg Cisco routers.
- *
- * Revision 1.44  2001/02/09 05:13:56  craigs
- * Added pragma implementation to (hopefully) reduce the executable image size
- * under Linux
- *
- * Revision 1.43  2001/01/04 07:46:22  robertj
- * Added special test for broken NetMeeting in restablishing logical channels
- *   when have to close one due to device conflict (asymetric codecs).
- *
- * Revision 1.42  2001/01/02 08:03:28  robertj
- * Changed MSD determination number comparision to be clearly as per spec.
- *
- * Revision 1.41  2000/10/16 09:46:37  robertj
- * Fixed problem with opening codec before capability fields updated from
- *    OLC PDU, caused problems in G.711 tweaked psuedo-packet size.
- *
- * Revision 1.40  2000/09/22 01:35:50  robertj
- * Added support for handling LID's that only do symmetric codecs.
- *
- * Revision 1.39  2000/09/12 01:01:17  robertj
- * Fixed protocol issue, should send CloseLogicalChannel PDU's on accepting RequestClose.
- *
- * Revision 1.38  2000/08/31 10:49:38  robertj
- * Fixed unix deadlock when non-fastStart but with H245 tunneling.
- *
- * Revision 1.37  2000/08/21 12:37:14  robertj
- * Fixed race condition if close call just as slow start media channels are opening, part 2.
- *
- * Revision 1.36  2000/08/21 02:50:28  robertj
- * Fixed race condition if close call just as slow start media channels are opening.
- *
- * Revision 1.35  2000/07/14 09:00:48  robertj
- * Fixed race condition in closing connection and explicit close logical channel.
- *
- * Revision 1.34  2000/07/11 19:31:28  robertj
- * Fixed possible deadlock under unix when trying to restart running TCS.
- *
- * Revision 1.33  2000/07/10 16:04:01  robertj
- * Added TCS=0 support.
- * Fixed bug where negotiations hang if not fast start and tunnelled but remot does not tunnel.
- *
- * Revision 1.32  2000/07/09 14:57:54  robertj
- * Fixed bug where don't accept TCS if supplied sequence number is zero.
- *
- * Revision 1.31  2000/07/04 13:00:36  craigs
- * Fixed problem with selecting large and small video sizes
- *
- * Revision 1.30  2000/07/04 04:15:41  robertj
- * Fixed capability check of "combinations" for fast start cases.
- *
- * Revision 1.29  2000/07/03 00:38:40  robertj
- * Fixed deadlock caused by nested mutex waits in negotiated channel closing.
- *
- * Revision 1.28  2000/06/27 03:46:52  robertj
- * Fixed another possible race condition on logical channel closure.
- *
- * Revision 1.27  2000/06/03 03:16:39  robertj
- * Fixed using the wrong capability table (should be connections) for some operations.
- *
- * Revision 1.26  2000/05/23 11:32:37  robertj
- * Rewrite of capability table to combine 2 structures into one and move functionality into that class
- *    allowing some normalisation of usage across several applications.
- * Changed H323Connection so gets a copy of capabilities instead of using endponts, allows adjustments
- *    to be done depending on the remote client application.
- *
- * Revision 1.25  2000/05/22 07:32:51  craigs
- * Fixed problem with ohphone silence detection hanging
- *
- * Revision 1.24  2000/05/16 08:13:42  robertj
- * Added function to find channel by session ID, supporting H323Connection::FindChannel() with mutex.
- *
- * Revision 1.23  2000/05/11 04:19:45  robertj
- * Fixed typo
- *
- * Revision 1.22  2000/05/11 04:16:35  robertj
- * Fixed missing timeout (and typo!) in bidirectional open logical channel.
- *
- * Revision 1.21  2000/05/08 05:06:27  robertj
- * Fixed bug in H.245 close logical channel timeout, thanks XuPeili.
- *
- * Revision 1.20  2000/05/05 04:35:10  robertj
- * Changed where the system timer is sampled to minimise overhead in round trip delay calculation.
- *
- * Revision 1.19  2000/05/02 04:32:27  robertj
- * Fixed copyright notice comment.
- *
- * Revision 1.18  2000/04/24 23:47:01  robertj
- * Fixed bug in master/slave negotiator for if have "in sync" random number generators, thanks Shawn Sincoski.
- *
- * Revision 1.17  2000/04/10 17:50:28  robertj
- * Fixed yet another race condition needing mutex in logical channels management class.
- *
- * Revision 1.16  2000/04/05 03:17:31  robertj
- * Added more RTP statistics gathering and H.245 round trip delay calculation.
- *
- * Revision 1.15  2000/03/29 02:14:46  robertj
- * Changed TerminationReason to CallEndReason to use correct telephony nomenclature.
- * Added CallEndReason for capability exchange failure.
- *
- * Revision 1.14  2000/03/25 02:19:50  robertj
- * Fixed missing mutex call in some logical channels structure access.
- *
- * Revision 1.13  2000/02/17 12:07:43  robertj
- * Used ne wPWLib random number generator after finding major problem in MSVC rand().
- *
- * Revision 1.12  1999/11/26 01:25:01  craigs
- * Removed deadlock when remote endpoint requests unsupported channel type
- *
- * Revision 1.11  1999/11/19 09:07:27  robertj
- * Fixed bug allocating incorrect channel number to fast start transmit channel.
- *
- * Revision 1.10  1999/11/06 11:00:18  robertj
- * Fixed race condition in explicit channel close and connection close.
- *
- * Revision 1.9  1999/11/06 05:37:45  robertj
- * Complete rewrite of termination of connection to avoid numerous race conditions.
- *
- * Revision 1.8  1999/10/30 12:34:47  robertj
- * Added information callback for closed logical channel on H323EndPoint.
- *
- * Revision 1.7  1999/10/24 04:45:14  robertj
- * Removed requirement that must be sending capability set to receive capability set.
- *
- * Revision 1.6  1999/09/21 14:10:41  robertj
- * Fixed incorrect PTRACING test and removed uneeded include of videoio.h
- *
- * Revision 1.5  1999/09/15 01:26:27  robertj
- * Changed capability set call backs to have more specific class as parameter.
- *
- * Revision 1.4  1999/09/14 14:26:17  robertj
- * Added more debug tracing.
- *
- * Revision 1.3  1999/09/08 04:05:49  robertj
- * Added support for video capabilities & codec, still needs the actual codec itself!
- *
- * Revision 1.2  1999/08/31 12:34:19  robertj
- * Added gatekeeper support.
- *
- * Revision 1.1  1999/08/25 05:08:14  robertj
- * File fission (critical mass reached).
+ * $Id: h323neg.cxx,v 1.13 2011/09/22 03:36:54 shorne Exp $
  *
  */
 
@@ -609,7 +372,14 @@ PBoolean H245NegTerminalCapabilitySet::HandleIncoming(const H245_TerminalCapabil
 
   mutex.Signal();
 
+  // give application a chance to inspect / modify received H.245 capabilities
+  connection.OnReceivedCapabilitySet(pdu);
+
+#ifdef H323_H235
+  H235Capabilities remoteCapabilities(connection, pdu);
+#else
   H323Capabilities remoteCapabilities(connection, pdu);
+#endif
 
   const H245_MultiplexCapability * muxCap = NULL;
   if (pdu.HasOptionalField(H245_TerminalCapabilitySet::e_multiplexCapability))
@@ -813,36 +583,36 @@ PBoolean H245NegLogicalChannel::OpenWhileLocked(const H323Capability & capabilit
       open.m_forwardLogicalChannelParameters.m_replacementFor = replacementFor;
     }
   }
-
+/*
 #ifdef H323_H239
   if (sessionID == OpalMediaFormat::DefaultExtVideoSessionID) {  // extended Video Session
 	open.IncludeOptionalField(H245_OpenLogicalChannel::e_genericInformation);
     H245_ArrayOf_GenericInformation & cape = open.m_genericInformation;
 
-	  H245_GenericMessage * gcap = new H245_GenericMessage();
-	  gcap->m_messageIdentifier = *(new H245_CapabilityIdentifier(H245_CapabilityIdentifier::e_standard));
-	  PASN_ObjectId &object_id = gcap->m_messageIdentifier;
+	  H245_GenericInformation gcap;
+	  gcap.m_messageIdentifier = *(new H245_CapabilityIdentifier(H245_CapabilityIdentifier::e_standard));
+	  PASN_ObjectId &object_id = gcap.m_messageIdentifier;
       object_id = OpalPluginCodec_Identifer_H239_Video;   // Indicates H239 (Extended Video)
 
-	  gcap->IncludeOptionalField(H245_GenericMessage::e_messageContent);
-		H245_ArrayOf_GenericParameter & params = gcap->m_messageContent;
-			H245_GenericParameter * content = new H245_GenericParameter();
-			H245_ParameterIdentifier & paramid = content->m_parameterIdentifier;
+	  gcap.IncludeOptionalField(H245_GenericMessage::e_messageContent);
+		H245_ArrayOf_GenericParameter & params = gcap.m_messageContent;
+		    params.SetSize(1);
+			H245_GenericParameter & content = params[0];
+			H245_ParameterIdentifier & paramid = content.m_parameterIdentifier;
 				paramid.SetTag(H245_ParameterIdentifier::e_standard);
 				PASN_Integer & pid = paramid;
 				pid.SetValue(1);
 
-				H245_ParameterValue & paramval = content->m_parameterValue;
+				H245_ParameterValue & paramval = content.m_parameterValue;
 				paramval.SetTag(H245_ParameterValue::e_booleanArray);
 				PASN_Integer & val = paramval;
 				val.SetValue(roleLabel);
-		params.Append(content);
-		params.SetSize(params.GetSize()+1);
-      cape.Append(gcap);
-      cape.SetSize(cape.GetSize()+1);
+	  PINDEX sz = cape.GetSize();
+	  cape.SetSize(sz+1);
+      cape[sz] = gcap; 
   }
 #endif
-
+*/
   if (!channel->Open())
     return FALSE;
 
@@ -906,10 +676,16 @@ PBoolean H245NegLogicalChannel::HandleOpen(const H245_OpenLogicalChannel & pdu)
   PBoolean ok = FALSE;
 
   unsigned cause = H245_OpenLogicalChannelReject_cause::e_unspecified;
-  if (connection.OnOpenLogicalChannel(pdu, ack, cause))
-    channel = connection.CreateLogicalChannel(pdu, FALSE, cause);
+    
+  channel = connection.CreateLogicalChannel(pdu, FALSE, cause);
 
   if (channel != NULL) {
+	  if (!connection.OnOpenLogicalChannel(pdu, ack, cause, channel->GetSessionID())) {
+		  delete channel;
+		  channel = NULL;
+		  ok = false;
+	  }
+
     channel->SetNumber(channelNumber);
     channel->OnSendOpenAck(pdu, ack);
     if (channel->GetDirection() == H323Channel::IsBidirectional) {
@@ -924,7 +700,8 @@ PBoolean H245NegLogicalChannel::HandleOpen(const H245_OpenLogicalChannel & pdu)
         // the master. However NetMeeting will not then reopen a channel, so
         // we act like we are a slave and close our end instead.
         if (connection.IsH245Master() &&
-            connection.GetRemoteApplication().Find("NetMeeting") == P_MAX_INDEX)
+            ((connection.GetRemoteApplication().Find("NetMeeting") == P_MAX_INDEX) ||
+			(connection.GetRemoteApplication().Find("Tandberg	27") == P_MAX_INDEX)))   // T1000
           cause = H245_OpenLogicalChannelReject_cause::e_masterSlaveConflict;
         else {
           connection.OnConflictingLogicalChannel(*channel);
@@ -944,7 +721,13 @@ PBoolean H245NegLogicalChannel::HandleOpen(const H245_OpenLogicalChannel & pdu)
     Release();
   }
 
-  return connection.WriteControlPDU(reply);
+  if (!connection.WriteControlPDU(reply))
+      return false;
+
+  if(ok)
+      return connection.OnInitialFlowRestriction(*channel);
+
+  return true;
 }
 
 
@@ -965,6 +748,15 @@ PBoolean H245NegLogicalChannel::HandleOpenAck(const H245_OpenLogicalChannelAck &
 
       if (!channel->OnReceivedAckPDU(pdu))
         return CloseWhileLocked();
+      
+      // If extended Video channel, then send Channel Active
+      if (channel->GetCapability().GetMainType() == H323Capability::e_Video && 
+		 channel->GetCapability().GetSubType() == H245_VideoCapability::e_extendedVideoCapability) {
+         H323ControlPDU reply;
+         reply.BuildLogicalChannelActive(channelNumber);
+         if (!connection.WriteControlPDU(reply))
+                 return FALSE;
+      }
 
       if (channel->GetDirection() == H323Channel::IsBidirectional) {
         H323ControlPDU reply;
@@ -1428,6 +1220,13 @@ H323ChannelNumber H245NegLogicalChannels::GetNextChannelNumber()
 }
 
 
+H323ChannelNumber H245NegLogicalChannels::GetLastChannelNumber()
+{
+  PWaitAndSignal wait(mutex);
+  return lastChannelNumber;
+}
+
+
 H323Channel * H245NegLogicalChannels::GetChannelAt(PINDEX i)
 {
   mutex.Wait();
