@@ -27,6 +27,27 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323pdu.cxx,v $
+ * Revision 1.14  2011/05/09 07:35:47  shorne
+ * updates for changes in PTLIB v2.11. The replacement of PDICTIONARY with H323Dictionary. Updates for the changes in the PNatMethod Class. Alteration in PFactory::Create, Remove redundent code and change change logging
+ *
+ * Revision 1.13  2011/03/24 13:17:51  willamowius
+ * avoid uninitialized memory access and minor speedup
+ *
+ * Revision 1.12  2011/02/20 06:55:46  shorne
+ * Fixes for H.460 to allow better selection of mesasage location in PDU. Features or Generic Data. Corrected H.460.9
+ *
+ * Revision 1.11  2011/01/12 13:05:04  shorne
+ * Added detection of keep-alive packets in signalling channel
+ *
+ * Revision 1.10  2010/08/31 04:00:49  shorne
+ * Improved H.263/H.263+ interworking
+ *
+ * Revision 1.9  2010/08/26 15:12:39  shorne
+ * Major H.239 upgrade. Special thx again to Marek Domaracky and Igor Pavlov
+ *
+ * Revision 1.8  2010/05/02 22:44:31  shorne
+ * Added support to be able to set NAT Method on a call by call basis
+ *
  * Revision 1.7  2008/09/27 06:18:57  shorne
  * BUG FIX: H323SignalPDU::Write to correctly handle NULL H323Connection Thx Nir Soffer
  *
@@ -683,7 +704,7 @@ void H323SetAliasAddresses(const PStringList & names,
 
 static PBoolean IsE164(const PString & str)
 {
-  return !str && strspn(str, "1234567890*#,") == strlen(str);
+  return !str.IsEmpty() && (str.FindSpan("1234567890*#,") == P_MAX_INDEX);
 }
 
 static PBoolean IsURL(const PString & str)
@@ -716,20 +737,19 @@ void H323SetAliasAddress(const PString & _name, H225_AliasAddress & alias, int t
   PString name = _name;
   // See if alias type was explicitly specified
   if (tag < 0) {
-    PINDEX colon = name.Find(':');
-    if (colon != P_MAX_INDEX && colon > 0) {
-      PString type = name.Left(colon);
-      for (PINDEX i = 0; tag < 0 && i < 5; i++) {
-        if (type == aliasAddressTypes[i].name) {
-          tag = aliasAddressTypes[i].tag;
-          name = name.Mid(colon+1);
+      PINDEX colon = name.Find(':');
+      if (colon != P_MAX_INDEX && colon > 0) {
+        PString type = name.Left(colon);
+        for (PINDEX i = 0; tag < 0 && i < 5; i++) {
+          if (type == aliasAddressTypes[i].name) {
+            tag = aliasAddressTypes[i].tag;
+            name = name.Mid(colon+1);
+          }
         }
       }
-    }
   }
-  
-  // otherwise guess it from the string: if all digits then assume an e164 address, @ URL else H323_ID.
-  if (tag < 0) {
+
+  if (tag < 0 || tag == 1) {
 	if (IsE164(name)) 
 		tag = H225_AliasAddress::e_dialedDigits;
 	else if (IsURL(name)) 
@@ -909,7 +929,7 @@ PBoolean H323SetRTPPacketization(H245_RTPPayloadType & rtpPacketization,
                              const OpalMediaFormat & mediaFormat,
                              RTP_DataFrame::PayloadTypes payloadType)
 {
-  PString mediaPacketization = mediaFormat.GetOptionString("Media Packetization");
+  PString mediaPacketization = mediaFormat.GetOptionString(PLUGINCODEC_MEDIA_PACKETIZATION);
   if (mediaPacketization.IsEmpty())
     return FALSE;
 
@@ -971,7 +991,7 @@ PBoolean H323GetRTPPacketization(OpalMediaFormat & mediaFormat, const H245_RTPPa
       return FALSE;
   }
   
-  mediaFormat.SetOptionString("Media Packetization", mediaPacketization);
+  mediaFormat.SetOptionString(PLUGINCODEC_MEDIA_PACKETIZATION, mediaPacketization);
   return TRUE;
 }
 
@@ -983,7 +1003,7 @@ static void SendSetupFeatureSet(const H323Connection * connection, H225_Setup_UU
    
   H225_FeatureSet fs;
 
-    if (!connection->OnSendFeatureSet(H460_MessageType::e_setup, fs)) 
+    if (!connection->OnSendFeatureSet(H460_MessageType::e_setup, fs, true)) 
 	      return;
 
 	if (fs.HasOptionalField(H225_FeatureSet::e_neededFeatures)) {
@@ -1010,13 +1030,13 @@ template <typename PDUType>
 static void SendFeatureSet(const H323Connection * connection, unsigned code, H225_H323_UU_PDU & msg, PDUType & pdu)
 {
  H225_FeatureSet fs;
-    if (!connection->OnSendFeatureSet(code,fs))
-		return;
 
-	if (code == H460_MessageType::e_callProceeding) {
+    if (connection->OnSendFeatureSet(code,fs,true)) {
         pdu.IncludeOptionalField(PDUType::e_featureSet);
 	    pdu.m_featureSet = fs; 
-    } else {
+    }
+
+    if (connection->OnSendFeatureSet(code,fs,false)) {
 		if (fs.HasOptionalField(H225_FeatureSet::e_supportedFeatures)) {
 			msg.IncludeOptionalField(H225_H323_UU_PDU::e_genericData);
 
@@ -1548,7 +1568,7 @@ void H323SignalPDU::BuildQ931()
 
 void H323SignalPDU::PrintOn(ostream & strm) const
 {
-  int indent = strm.precision() + 2;
+  int indent = (int)strm.precision() + 2;
   strm << "{\n"
        << setw(indent+10) << "q931pdu = " << setprecision(indent) << q931pdu << '\n'
        << setw(indent+10) << "h225pdu = " << setprecision(indent);
@@ -1573,6 +1593,11 @@ PBoolean H323SignalPDU::Read(H323Transport & transport)
 
 PBoolean H323SignalPDU::ProcessReadData(H323Transport & transport, const PBYTEArray & rawData)
 {
+  if (rawData.GetSize() < 5) {
+     PTRACE(4,"H225\tSignalling Channel KeepAlive Rec'vd");
+     return TRUE;
+  }
+
   if (!q931pdu.Decode(rawData)) {
     PTRACE(1, "H225\tParse error of Q931 PDU:\n" << hex << setfill('0')
                                                  << setprecision(2) << rawData
@@ -2005,7 +2030,6 @@ H245_TerminalCapabilitySet &
   return cap;
 }
 
-
 H245_TerminalCapabilitySetAck &
       H323ControlPDU::BuildTerminalCapabilitySetAck(unsigned sequenceNumber)
 {
@@ -2086,6 +2110,16 @@ H245_OpenLogicalChannelConfirm &
   return chan;
 }
 
+H245_IndicationMessage &
+     H323ControlPDU::BuildLogicalChannelActive(unsigned channelNumber)
+{
+   H245_IndicationMessage & lca = Build(H245_IndicationMessage::e_miscellaneousIndication);
+   H245_MiscellaneousIndication & miscIndication = lca;
+   miscIndication.m_logicalChannelNumber = channelNumber;
+   miscIndication.m_type.SetTag(H245_MiscellaneousIndication_type::e_logicalChannelActive);
+   return lca;
+}
+
 
 H245_CloseLogicalChannelAck &
       H323ControlPDU::BuildCloseLogicalChannelAck(unsigned channelNumber)
