@@ -24,6 +24,12 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h450pdu.cxx,v $
+ * Revision 1.4  2010/08/11 17:04:49  willamowius
+ * someone needs to check this alias type
+ *
+ * Revision 1.3  2010/06/24 14:13:05  willamowius
+ * fix crash if H.450.2 transfer destination can't be resolved
+ *
  * Revision 1.2  2008/05/23 11:23:11  willamowius
  * switch BOOL to PBoolean to be able to compile with Ptlib 2.2.x
  *
@@ -227,7 +233,7 @@ void H450ServiceAPDU::BuildCallTransferInitiate(int invokeId,
     aliasAddress.SetSize(2);
 
     // Set the alias
-    aliasAddress[1].SetTag(H225_AliasAddress::e_dialedDigits);
+    aliasAddress[1].SetTag(H225_AliasAddress::e_dialedDigits);	// TODO: can't this also be another alias type ???
     H323SetAliasAddress(alias, aliasAddress[1]);
 
     // Set the transport
@@ -1399,7 +1405,10 @@ void H4502Handler::TransferCall(const PString & remoteParty,
 
   PStringList Addresses;
   endpoint.ResolveCallParty(remoteParty, Addresses);
-  endpoint.ParsePartyName(Addresses[0], alias, address);
+  if (Addresses.GetSize() > 0)
+    endpoint.ParsePartyName(Addresses[0], alias, address);
+  else
+    PTRACE(1, "H4502\tCould not resolve transfer destination " << remoteParty);
 
   serviceAPDU.BuildCallTransferInitiate(currentInvokeId, callIdentity, alias, address);
   serviceAPDU.WriteFacilityPDU(connection);
