@@ -27,6 +27,18 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: q931.cxx,v $
+ * Revision 1.7  2011/04/08 17:10:18  willamowius
+ * fix compilation with PTLib 2.11.x
+ *
+ * Revision 1.6  2010/05/02 22:53:52  shorne
+ * Fix warning on vs2010
+ *
+ * Revision 1.5  2010/04/30 19:17:18  willamowius
+ * fix memory leak when q931 decoding fails
+ *
+ * Revision 1.4  2010/01/04 05:54:47  shorne
+ * Added SetCallingSubAddressIE and SetCalledSubAddressIE support
+ *
  * Revision 1.3  2008/05/27 03:15:20  shorne
  * Updated Windows compilation to compile with latest ptlib SVN
  *
@@ -256,6 +268,7 @@
 #endif
 
 #include "q931.h"
+#include "openh323buildopts.h"
 
 #include <ptclib/random.h>
 
@@ -578,16 +591,20 @@ PBoolean Q931::Decode(const PBYTEArray & data)
         // we also have a protocol discriminator, which we ignore
         offset++;
 
-	// before decrementing the length, make sure it is not zero
-	if (len == 0)
+        // before decrementing the length, make sure it is not zero
+        if (len == 0) {
+          delete item;
           return FALSE;
+        }
 
         // adjust for protocol discriminator
         len--;
       }
 
-      if (offset + len > data.GetSize())
+      if (offset + len > data.GetSize()) {
+        delete item;
         return FALSE;
+      }
 
       memcpy(item->GetPointer(len), (const BYTE *)data+offset, len);
       offset += len;
@@ -664,7 +681,7 @@ PBoolean Q931::Encode(PBYTEArray & data) const
 
 void Q931::PrintOn(ostream & strm) const
 {
-  int indent = strm.precision() + 2;
+  int indent = (int)strm.precision() + 2;
   _Ios_Fmtflags flags = strm.flags();
 
   strm << "{\n"
@@ -1189,7 +1206,11 @@ static PBoolean GetNumberIE(const PBYTEArray & bytes,
   PINDEX len = bytes.GetSize()-offset;
 
   if (len > 0)
+#if PTLIB_VER >= 2110
+    memcpy(number.GetPointerAndSetLength(len), ((const BYTE *)bytes)+offset, len);
+#else
     memcpy(number.GetPointer(len+1), ((const BYTE *)bytes)+offset, len);
+#endif
 
   return !number;
 }
@@ -1219,6 +1240,17 @@ PBoolean Q931::GetCallingPartyNumber(PString  & number,
                      defPresentation, defScreening, 0);
 }
 
+void Q931::SetCallingPartySubAddress(const PString & number, unsigned plan, unsigned type)
+{
+  SetIE(CallingPartySubAddressIE,
+        SetNumberIE(number, plan, type, -1, -1, -1));
+}
+
+PBoolean Q931::GetCallingPartySubAddress(PString & number, unsigned * plan, unsigned * type) const
+{
+  return GetNumberIE(GetIE(CallingPartySubAddressIE),
+                     number, plan, type, NULL, NULL, NULL, 0, 0, 0);
+}
 
 void Q931::SetCalledPartyNumber(const PString & number, unsigned plan, unsigned type)
 {
@@ -1233,6 +1265,17 @@ PBoolean Q931::GetCalledPartyNumber(PString & number, unsigned * plan, unsigned
                      number, plan, type, NULL, NULL, NULL, 0, 0, 0);
 }
 
+void Q931::SetCalledPartySubAddress(const PString & number, unsigned plan, unsigned type)
+{
+  SetIE(CalledPartySubAddressIE,
+        SetNumberIE(number, plan, type, -1, -1, -1));
+}
+
+PBoolean Q931::GetCalledPartySubAddress(PString & number, unsigned * plan, unsigned * type) const
+{
+  return GetNumberIE(GetIE(CalledPartySubAddressIE),
+                     number, plan, type, NULL, NULL, NULL, 0, 0, 0);
+}
 
 void Q931::SetRedirectingNumber(const PString & number,
                                 unsigned plan,
