@@ -3,7 +3,7 @@
  *
  * RTP protocol handler
  *
- * Open H323 Library
+ * H323plus Library
  *
  * Copyright (c) 1998-2000 Equivalence Pty. Ltd.
  *
@@ -26,385 +26,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: rtp.cxx,v $
- * Revision 1.6  2008/10/28 23:06:28  willamowius
- * fixes to compile with audio disabled
- *
- * Revision 1.5  2008/09/27 06:20:26  shorne
- * BUG FIX: RTP_UDP::Write now returns false if there is an error writing to the control channel
- *
- * Revision 1.4  2008/05/23 11:22:34  willamowius
- * switch BOOL to PBoolean to be able to compile with Ptlib 2.2.x
- *
- * Revision 1.3  2008/02/06 02:52:59  shorne
- * Added support for Standards based NAT Traversal
- *
- * Revision 1.2  2007/10/16 17:03:54  shorne
- * Qos capability negotiation
- *
- * Revision 1.1  2007/08/06 20:51:07  shorne
- * First commit of h323plus
- *
- * Revision 1.106.2.5  2007/08/02 20:30:44  shorne
- * Added trace to determine which NAT method opened what ports
- *
- * Revision 1.106.2.4  2007/07/23 21:47:11  shorne
- * Added QoS GK Reporting
- *
- * Revision 1.106.2.3  2007/05/23 07:02:38  shorne
- * Removed VS 2005 compile warning
- *
- * Revision 1.106.2.2  2007/02/17 15:26:20  shorne
- * Support static memory allocation of RTP_DataFrame for Plugin Video Support
- *
- * Revision 1.106.2.1  2006/12/23 19:08:03  shorne
- * Plugin video codecs & sundry
- *
- * Revision 1.106  2006/01/26 03:35:04  shorne
- * Added more NAT traversal support
- *
- * Revision 1.105  2006/01/18 07:46:08  csoutheren
- * Initial version of RTP aggregation (disabled by default)
- *
- * Revision 1.104  2005/11/21 21:06:23  shorne
- * Added GQoS switches
- *
- * Revision 1.103  2005/08/27 02:06:01  csoutheren
- * Keep track of time first RTP packet is received
- *
- * Revision 1.102  2005/01/16 20:39:44  csoutheren
- * Fixed problem with IPv6 INADDR_ANY
- *
- * Revision 1.101  2005/01/04 08:08:46  csoutheren
- * More changes to implement the new configuration methodology, and also to
- * attack the global static problem
- *
- * Revision 1.100  2005/01/03 14:03:42  csoutheren
- * Added new configure options and ability to disable/enable modules
- *
- * Revision 1.99  2005/01/03 06:26:09  csoutheren
- * Added extensive support for disabling code modules at compile time
- *
- * Revision 1.98  2004/07/03 06:51:37  rjongbloed
- * Added PTRACE_PARAM() macro to fix warnings on parameters used in PTRACE
- *  macros only.
- *
- * Revision 1.97  2004/05/02 18:49:08  ykiryanov
- * Removed ifdef BeOS to add functionality
- *
- * Revision 1.96  2004/04/24 23:58:05  rjongbloed
- * Fixed GCC 3.4 warning about PAssertNULL
- *
- * Revision 1.95  2004/04/22 12:38:04  rjongbloed
- * Removed the RTP QoS class if there is no QoS support in build,
- *   thanks Nick Hoath, ISDN Communications Ltd.
- *
- * Revision 1.94  2004/04/07 05:31:43  csoutheren
- * Added ability to receive calls from endpoints behind NAT firewalls
- *
- * Revision 1.93  2004/03/02 10:02:13  rjongbloed
- * Added check for unusual error in reading UDP socket, thanks Michael Smith
- *
- * Revision 1.92  2004/02/09 11:17:50  rjongbloed
- * Improved check for compound RTCP packets so does not possibly acess
- *   memory beyond that allocated in packet. Pointed out by Paul Slootman
- *
- * Revision 1.91  2003/10/28 22:35:21  dereksmithies
- * Fix warning about possible use of unitialized variable.
- *
- * Revision 1.90  2003/10/27 06:03:39  csoutheren
- * Added support for QoS
- *   Thanks to Henry Harrison of AliceStreet
- *
- * Revision 1.89  2003/10/09 09:47:45  csoutheren
- * Fixed problem with re-opening RTP half-channels under unusual
- * circumstances. Thanks to Damien Sandras
- *
- * Revision 1.88  2003/05/02 04:57:47  robertj
- * Added header extension support to RTP data frame class.
- *
- * Revision 1.87  2003/02/07 00:30:21  robertj
- * Changes for bizarre usage of RTP code outside of scope of H.323 specs.
- *
- * Revision 1.86  2003/02/04 23:50:06  robertj
- * Changed trace log for RTP session creation to show local address.
- *
- * Revision 1.85  2003/02/04 07:06:42  robertj
- * Added STUN support.
- *
- * Revision 1.84  2003/01/07 06:32:22  robertj
- * Fixed faint possibility of getting an error on UDP write caused by ICMP
- *   unreachable being received when no UDP read is active on the socket. Then
- *   the UDP write gets the error stopping transmission.
- *
- * Revision 1.83  2002/11/19 01:48:00  robertj
- * Allowed get/set of canonical anme and tool name.
- *
- * Revision 1.82  2002/11/05 03:32:04  robertj
- * Added session ID to trace logs.
- * Fixed possible extra wait exiting RTP read loop on close.
- *
- * Revision 1.81  2002/11/05 01:55:50  robertj
- * Added comments about strange mutex usage.
- *
- * Revision 1.80  2002/10/31 00:47:07  robertj
- * Enhanced jitter buffer system so operates dynamically between minimum and
- *   maximum values. Altered API to assure app writers note the change!
- *
- * Revision 1.79  2002/09/26 04:01:49  robertj
- * Fixed calculation of fraction of packets lost in RR, thanks Awais Ali
- *
- * Revision 1.78  2002/09/03 06:15:32  robertj
- * Added copy constructor/operator for session manager.
- *
- * Revision 1.77  2002/08/05 10:03:48  robertj
- * Cosmetic changes to normalise the usage of pragma interface/implementation.
- *
- * Revision 1.76  2002/07/23 06:28:16  robertj
- * Added statistics call back on first sent or received RTP packet, helps with,
- *   for example, detecting if remote endpoint has started to send audio.
- *
- * Revision 1.75  2002/05/28 02:37:55  robertj
- * Fixed reading data out of RTCP compound statements.
- *
- * Revision 1.74  2002/05/21 07:12:39  robertj
- * Fixed 100% CPU usage loop for checkin on sending RTCP data if
- *   have no RTP data transferred at all.
- *
- * Revision 1.73  2002/05/02 05:58:28  robertj
- * Changed the mechanism for sending RTCP reports so that they will continue
- *   to be sent regardless of if there is any actual data traffic.
- * Added support for compound RTCP statements for sender and receiver reports.
- *
- * Revision 1.72  2002/04/18 05:48:58  robertj
- * Fixed problem with new SSRC value being ignored after RTP session has
- *    been restarted, thanks "Jacky".
- *
- * Revision 1.71  2002/02/09 02:33:49  robertj
- * Improved payload type docuemntation and added Cisco CN.
- *
- * Revision 1.70  2001/12/20 04:34:56  robertj
- * Fixed display of some of the unknown RTP types.
- *
- * Revision 1.69  2001/09/12 07:48:05  robertj
- * Fixed various problems with tracing.
- *
- * Revision 1.68  2001/09/11 00:21:24  robertj
- * Fixed missing stack sizes in endpoint for cleaner thread and jitter thread.
- *
- * Revision 1.67  2001/09/10 08:24:04  robertj
- * Fixed setting of destination RTP address so works with endpoints that
- *   do not get the OLC packets quite right.
- *
- * Revision 1.66  2001/07/11 03:23:54  robertj
- * Bug fixed where every 65536 PDUs the session thinks it is the first PDU again.
- *
- * Revision 1.65  2001/07/06 06:32:24  robertj
- * Added flag and checks for RTP data having specific SSRC.
- * Changed transmitter IP address check so is based on first received
- *    PDU instead of expecting it to come from the host we are sending to.
- *
- * Revision 1.64  2001/06/04 13:43:44  robertj
- * Fixed I/O block breaker code if RTP session is bound to INADDR_ANY interface.
- *
- * Revision 1.63  2001/06/04 11:37:50  robertj
- * Added thread safe enumeration functions of RTP sessions.
- * Added member access functions to UDP based RTP sessions.
- *
- * Revision 1.62  2001/05/24 01:12:23  robertj
- * Fixed sender report timestamp field, thanks Johan Gnosspelius
- *
- * Revision 1.61  2001/05/08 05:28:02  yurik
- * No ifdef _WIN32_WCE anymore - 3+ version of  SDK allows it
- *
- * Revision 1.60  2001/04/24 06:15:50  robertj
- * Added work around for strange Cisco bug which suddenly starts sending
- *   RTP packets beginning at a difference sequence number base.
- *
- * Revision 1.59  2001/04/02 23:58:24  robertj
- * Added jitter calculation to RTP session.
- * Added trace of statistics.
- *
- * Revision 1.58  2001/03/20 07:24:05  robertj
- * Changed RTP SDES to have simple host name instead of attempting a
- *   reverse DNS lookup on IP address as badly configured DNS can cause
- *   a 30 second delay before audio is sent.
- *
- * Revision 1.57  2001/03/15 05:45:45  robertj
- * Changed, yet again, the setting of RTP info to allow for Cisco idosyncracies.
- *
- * Revision 1.56  2001/02/21 08:08:26  robertj
- * Added more debugging.
- *
- * Revision 1.55  2001/02/09 05:13:56  craigs
- * Added pragma implementation to (hopefully) reduce the executable image size
- * under Linux
- *
- * Revision 1.54  2001/01/28 07:06:59  yurik
- * WinCE-port - avoid using of IP_TOS
- *
- * Revision 1.53  2000/12/18 08:59:20  craigs
- * Added ability to set ports
- *
- * Revision 1.52  2000/09/25 22:27:40  robertj
- * Fixed uninitialised variable for lastRRSequenceNumber
- *
- * Revision 1.51  2000/09/25 01:53:20  robertj
- * Fixed MSVC warnings.
- *
- * Revision 1.50  2000/09/25 01:44:13  robertj
- * Fixed possible race condition on shutdown of RTP session with jitter buffer.
- *
- * Revision 1.49  2000/09/22 00:32:34  craigs
- * Added extra logging
- * Fixed problems with no fastConnect with tunelling
- *
- * Revision 1.48  2000/09/21 02:06:07  craigs
- * Added handling for endpoints that return conformant, but useless, RTP address
- * and port numbers
- *
- * Revision 1.47  2000/08/17 00:42:39  robertj
- * Fixed RTP Goodbye message reason string parsing, thanks Thien Nguyen.
- *
- * Revision 1.46  2000/05/30 10:35:41  robertj
- * Fixed GNU compiler warning.
- *
- * Revision 1.45  2000/05/30 06:52:26  robertj
- * Fixed problem with Cisco restarting sequence numbers when changing H.323 logical channels.
- *
- * Revision 1.44  2000/05/23 12:57:37  robertj
- * Added ability to change IP Type Of Service code from applications.
- *
- * Revision 1.43  2000/05/12 00:27:35  robertj
- * Fixed bug in UseSession() that caused asserts on BSD and possible race condition everywhere.
- *
- * Revision 1.42  2000/05/04 11:52:35  robertj
- * Added Packets Too Late statistics, requiring major rearrangement of jitter
- *    buffer code, not also changes semantics of codec Write() function slightly.
- *
- * Revision 1.41  2000/05/02 04:32:27  robertj
- * Fixed copyright notice comment.
- *
- * Revision 1.40  2000/05/01 01:01:49  robertj
- * Added flag for what to do with out of orer packets (use if jitter, don't if not).
- *
- * Revision 1.39  2000/04/30 03:55:09  robertj
- * Improved the RTCP messages, epecially reports
- *
- * Revision 1.38  2000/04/28 12:56:39  robertj
- * Fixed transmission of SDES record in RTCP channel.
- *
- * Revision 1.37  2000/04/19 01:50:05  robertj
- * Improved debugging messages.
- *
- * Revision 1.36  2000/04/13 18:07:39  robertj
- * Fixed missing mutex release causing possible deadlocks.
- *
- * Revision 1.35  2000/04/10 17:40:05  robertj
- * Fixed debug output of RTP payload types to allow for unknown numbers.
- *
- * Revision 1.34  2000/04/05 04:09:24  robertj
- * Fixed portability problem with max() macro.
- *
- * Revision 1.33  2000/04/05 03:17:32  robertj
- * Added more RTP statistics gathering and H.245 round trip delay calculation.
- *
- * Revision 1.32  2000/04/03 18:15:44  robertj
- * Added "fractional" part of RTCP status NTP timestamp field.
- *
- * Revision 1.31  2000/03/23 02:54:57  robertj
- * Added sending of SDES control packets.
- *
- * Revision 1.30  2000/03/20 20:53:42  robertj
- * Fixed problem with being able to reopen for reading an RTP_Session (Cisco compatibilty)
- *
- * Revision 1.29  2000/03/04 12:32:23  robertj
- * Added setting of TOS field in IP header to get poor mans QoS on some routers.
- *
- * Revision 1.28  2000/02/29 13:00:13  robertj
- * Added extra statistic display for RTP packets out of order.
- *
- * Revision 1.27  2000/02/29 02:13:56  robertj
- * Fixed RTP receive of both control and data, ignores ECONNRESET/ECONNREFUSED errors.
- *
- * Revision 1.26  2000/02/27 10:56:24  robertj
- * Fixed error in code allowing non-consecutive RTP port numbers coming from broken stacks, thanks Vassili Leonov.
- *
- * Revision 1.25  2000/02/17 12:07:44  robertj
- * Used ne wPWLib random number generator after finding major problem in MSVC rand().
- *
- * Revision 1.24  2000/01/29 07:10:20  robertj
- * Fixed problem with RTP transmit to host that is not yet ready causing the
- *   receive side to die with ECONNRESET, thanks Ian MacDonald
- *
- * Revision 1.23  2000/01/20 05:57:46  robertj
- * Added extra flexibility in receiving incorrectly formed OpenLogicalChannel PDU's
- *
- * Revision 1.22  2000/01/14 00:31:40  robertj
- * Bug fix for RTP port allocation (MSVC optimised version), Thanks to Ian MacDonald.
- *
- * Revision 1.21  1999/12/30 09:14:49  robertj
- * Changed payload type functions to use enum.
- *
- * Revision 1.20  1999/12/23 23:02:36  robertj
- * File reorganision for separating RTP from H.323 and creation of LID for VPB support.
- *
- * Revision 1.19  1999/11/29 04:50:11  robertj
- * Added adaptive threshold calculation to silence detection.
- *
- * Revision 1.18  1999/11/22 00:09:59  robertj
- * Fixed error in RTP transmit rewrite for ternary state, didn't transmit at all!
- *
- * Revision 1.17  1999/11/20 05:35:48  robertj
- * Fixed possibly I/O block in RTP read loops.
- *
- * Revision 1.16  1999/11/19 13:13:31  robertj
- * Fixed Windows 95 compatibility issue in shutdown of RTP reading.
- *
- * Revision 1.15  1999/11/19 10:23:16  robertj
- * Fixed local binding of socket to correct address on multi-homes systems.
- *
- * Revision 1.14  1999/11/19 09:17:15  robertj
- * Fixed problems with aycnhronous shut down of logical channels.
- *
- * Revision 1.13  1999/11/17 03:49:51  robertj
- * Added RTP statistics display.
- *
- * Revision 1.12  1999/11/14 11:41:07  robertj
- * Added access functions to RTP statistics.
- *
- * Revision 1.11  1999/10/18 23:55:11  robertj
- * Fixed bug in setting contributing sources, length put into wrong spot in PDU header
- *
- * Revision 1.10  1999/09/21 14:10:04  robertj
- * Removed warnings when no tracing enabled.
- *
- * Revision 1.9  1999/09/05 00:58:37  robertj
- * Removed requirement that OpenLogicalChannalAck sessionId match original OLC.
- *
- * Revision 1.8  1999/09/03 02:17:50  robertj
- * Added more debugging
- *
- * Revision 1.7  1999/08/31 12:34:19  robertj
- * Added gatekeeper support.
- *
- * Revision 1.6  1999/07/16 02:13:54  robertj
- * Slowed down the control channel statistics packets.
- *
- * Revision 1.5  1999/07/13 09:53:24  robertj
- * Fixed some problems with jitter buffer and added more debugging.
- *
- * Revision 1.4  1999/07/09 06:09:52  robertj
- * Major implementation. An ENORMOUS amount of stuff added everywhere.
- *
- * Revision 1.3  1999/06/22 13:49:40  robertj
- * Added GSM support and further RTP protocol enhancements.
- *
- * Revision 1.2  1999/06/14 08:42:52  robertj
- * Fixed bug in dropped packets display was negative.
- *
- * Revision 1.1  1999/06/14 06:12:25  robertj
- * Changes for using RTP sessions correctly in H323 Logical Channel context
+ * $Id: rtp.cxx,v 1.31 2011/10/18 13:14:36 shorne Exp $
  *
  */
 
@@ -423,7 +45,7 @@
 #include "rtp.h"
 #include "h323con.h"
 
-#ifndef NO_H323_AUDIO_CODECS
+#ifdef H323_AUDIO_CODECS
 #include "jitter.h"
 #endif
 
@@ -619,6 +241,53 @@ ostream & operator<<(ostream & o, RTP_DataFrame::PayloadTypes t)
 
 #endif
 
+/////////////////////////////////////////////////////////////////////////////
+
+RTP_MultiDataFrame::RTP_MultiDataFrame(const BYTE * buffer, PINDEX length)
+: PBYTEArray(buffer,length)
+{
+}
+
+RTP_MultiDataFrame::RTP_MultiDataFrame(DWORD id, const BYTE * buffer, PINDEX rtplen)
+: PBYTEArray(rtplen+4)
+{
+   memcpy(theArray+4,buffer,rtplen);
+   *(PUInt32b *)&theArray[0] = id;
+}
+
+RTP_MultiDataFrame::RTP_MultiDataFrame(PINDEX rtplen)
+: PBYTEArray(rtplen+4)
+{
+}
+
+int  RTP_MultiDataFrame::GetMultiHeaderSize() const
+{
+    return 4;
+}
+
+DWORD RTP_MultiDataFrame::GetMultiplexID() const
+{
+   return *(PUInt32b *)&theArray[0];
+}
+
+void RTP_MultiDataFrame::SetMulitplexID(DWORD id)
+{
+   *(PUInt32b *)&theArray[0] = id;
+}
+
+void RTP_MultiDataFrame::GetRTPPayload(RTP_DataFrame & frame) const
+{
+     int sz = GetSize()- GetMultiHeaderSize();
+     frame.SetPayloadSize(sz - frame.GetHeaderSize());
+     memcpy(theArray+GetMultiHeaderSize(), frame.GetPointer(), sz);
+}
+
+void RTP_MultiDataFrame::SetRTPPayload(RTP_DataFrame & frame)
+{
+    int sz = frame.GetPayloadSize() + frame.GetHeaderSize();
+    SetSize(sz+ GetMultiHeaderSize());
+    memcpy(frame.GetPointer(),theArray+GetMultiHeaderSize(), sz);
+}
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -724,6 +393,47 @@ void RTP_ControlFrame::ReceiverReport::SetLostPackets(unsigned packets)
   lost[2] = (BYTE)packets;
 }
 
+/////////////////////////////////////////////////////////////////////////////
+
+RTP_MultiControlFrame::RTP_MultiControlFrame(BYTE const * buffer, PINDEX length)
+: PBYTEArray(buffer,length)
+{
+}
+
+RTP_MultiControlFrame::RTP_MultiControlFrame(PINDEX rtplen)
+: PBYTEArray(rtplen+4)
+{
+}
+
+int  RTP_MultiControlFrame::GetMultiHeaderSize() const
+{
+    return 4;
+}
+
+WORD RTP_MultiControlFrame::GetMultiplexID() const
+{
+   return *(PUInt16b *)&theArray[0];
+}
+
+void RTP_MultiControlFrame::SetMulitplexID(WORD id)
+{
+   *(PUInt16b *)&theArray[0] = id;
+}
+
+void RTP_MultiControlFrame::GetRTCPPayload(RTP_ControlFrame & frame) const
+{
+     int sz = GetSize()- GetMultiHeaderSize();
+     frame.SetPayloadSize(sz);
+     memcpy(theArray+GetMultiHeaderSize(), frame.GetPointer(), sz);
+}
+
+void RTP_MultiControlFrame::SetRTCPPayload(RTP_ControlFrame & frame)
+{
+    int sz = frame.GetSize();
+    SetSize(sz+ GetMultiHeaderSize());
+    memcpy(frame.GetPointer(), theArray+GetMultiHeaderSize(), sz);
+}
+
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -740,6 +450,13 @@ void RTP_UserData::OnFinalStatistics(const RTP_Session & /*session*/ ) const
 {
 }
 
+void RTP_UserData::OnRxSenderReport(unsigned /*sessionID*/,
+                                    const RTP_Session::SenderReport & /*send*/,
+                                    const RTP_Session::ReceiverReportArray & /*recv*/
+                                    ) const
+{
+}
+
 /////////////////////////////////////////////////////////////////////////////
 
 RTP_Session::RTP_Session(
@@ -757,13 +474,17 @@ RTP_Session::RTP_Session(
     ,aggregator(_aggregator)
 #endif
 {
-  PAssert(id > 0 && id < 256, PInvalidParameter);
   sessionID = (BYTE)id;
+  if (sessionID <= 0) {
+      PTRACE(2,"RTP\tWARNING: Session ID <= 0 Invalid SessionID.");
+  } else if (sessionID > 256) {
+      PTRACE(2,"RTP\tWARNING: Session ID " << sessionID << " Invalid SessionID.");
+  }
 
   referenceCount = 1;
   userData = data;
 
-#ifndef NO_H323_AUDIO_CODECS
+#ifdef H323_AUDIO_CODECS
   jitter = NULL;
 #endif
 
@@ -771,8 +492,8 @@ RTP_Session::RTP_Session(
   ignoreOutOfOrderPackets = TRUE;
   syncSourceOut = PRandom::Number();
   syncSourceIn = 0;
-  txStatisticsInterval = 100;  // Number of data packets between tx reports
-  rxStatisticsInterval = 100;  // Number of data packets between rx reports
+  txStatisticsInterval = 50; //100;  // Number of data packets between tx reports
+  rxStatisticsInterval = 50; //100;  // Number of data packets between rx reports
   lastSentSequenceNumber = (WORD)PRandom::Number();
   expectedSequenceNumber = 0;
   lastRRSequenceNumber = 0;
@@ -807,6 +528,7 @@ RTP_Session::RTP_Session(
   localAddress = PString();
   remoteAddress = PString(); 
 
+  avSyncData = false;
 }
 
 
@@ -815,7 +537,7 @@ RTP_Session::~RTP_Session()
   userData->OnFinalStatistics(*this);
 
   PTRACE_IF(2, packetsSent != 0 || packetsReceived != 0,
-            "RTP\tFinal statistics:\n"
+            "RTP\tFinal statistics: Session " << sessionID << "\n"
             "    packetsSent       = " << packetsSent << "\n"
             "    octetsSent        = " << octetsSent << "\n"
             "    averageSendTime   = " << averageSendTime << "\n"
@@ -834,11 +556,15 @@ RTP_Session::~RTP_Session()
             );
   delete userData;
 
-#ifndef NO_H323_AUDIO_CODECS
+#ifdef H323_AUDIO_CODECS
   delete jitter;
 #endif
 }
 
+void RTP_Session::SetSessionID(unsigned id)
+{
+    sessionID = id;
+}
 
 PString RTP_Session::GetCanonicalName() const
 {
@@ -884,19 +610,19 @@ void RTP_Session::SetJitterBufferSize(unsigned minJitterDelay,
                                       PINDEX stackSize)
 {
   if (minJitterDelay == 0 && maxJitterDelay == 0) {
-#ifndef NO_H323_AUDIO_CODECS
+#ifdef H323_AUDIO_CODECS
     delete jitter;
     jitter = NULL;
 #endif
   }
-#ifndef NO_H323_AUDIO_CODECS
+#ifdef H323_AUDIO_CODECS
   else if (jitter != NULL) {
     jitter->SetDelay(minJitterDelay, maxJitterDelay);
   }
 #endif
   else {
     SetIgnoreOutOfOrderPackets(FALSE);
-#ifndef NO_H323_AUDIO_CODECS
+#ifdef H323_AUDIO_CODECS
     jitter = new RTP_JitterBuffer(*this, minJitterDelay, maxJitterDelay, stackSize);
     jitter->Resume(
 #ifdef H323_RTP_AGGREGATE
@@ -911,7 +637,7 @@ void RTP_Session::SetJitterBufferSize(unsigned minJitterDelay,
 unsigned RTP_Session::GetJitterBufferSize() const
 {
   return
-#ifndef NO_H323_AUDIO_CODECS
+#ifdef H323_AUDIO_CODECS
   jitter != NULL ? jitter->GetJitterTime() :
 #endif
   0;
@@ -920,7 +646,7 @@ unsigned RTP_Session::GetJitterBufferSize() const
 
 PBoolean RTP_Session::ReadBufferedData(DWORD timestamp, RTP_DataFrame & frame)
 {
-#ifndef NO_H323_AUDIO_CODECS
+#ifdef H323_AUDIO_CODECS
   if (jitter != NULL)
     return jitter->ReadData(timestamp, frame);
   else
@@ -928,6 +654,20 @@ PBoolean RTP_Session::ReadBufferedData(DWORD timestamp, RTP_DataFrame & frame)
     return ReadData(frame, TRUE);
 }
 
+PBoolean RTP_Session::PseudoRead(int & /*selectStatus*/)
+{
+    return false;
+}
+
+bool RTP_Session::AVSyncData(SenderReport & sender)
+{
+    if (avSyncData) {
+        sender = rtpSync;
+        avSyncData = false;
+        return true;
+    }
+    return false;
+}
 
 void RTP_Session::SetTxStatisticsInterval(unsigned packets)
 {
@@ -1139,8 +879,8 @@ RTP_Session::SendReceiveStatus RTP_Session::OnReceiveData(const RTP_DataFrame &
   packetsReceived++;
 
   if (rtp.GetRemoteDataPort() > 0 && localAddress.IsEmpty()) {
-	  localAddress = rtp.GetLocalAddress().AsString() + ":" + PString(rtp.GetLocalDataPort());
-	  remoteAddress = rtp.GetRemoteAddress().AsString() + ":" + PString(rtp.GetRemoteDataPort());
+      localAddress = rtp.GetLocalAddress().AsString() + ":" + PString(rtp.GetLocalDataPort());
+      remoteAddress = rtp.GetRemoteAddress().AsString() + ":" + PString(rtp.GetRemoteDataPort());
   }
 
   // Call the statistics call-back on the first PDU with total count == 1
@@ -1288,7 +1028,7 @@ RTP_Session::SendReceiveStatus RTP_Session::OnReceiveControl(RTP_ControlFrame &
 
     switch (frame.GetPayloadType()) {
       case RTP_ControlFrame::e_SenderReport :
-        if (size >= sizeof(RTP_ControlFrame::SenderReport)) {
+        if (size >= (sizeof(RTP_ControlFrame::SenderReport)+frame.GetCount()*sizeof(RTP_ControlFrame::ReceiverReport))) {
           SenderReport sender;
           const RTP_ControlFrame::SenderReport & sr = *(const RTP_ControlFrame::SenderReport *)payload;
           sender.sourceIdentifier = sr.ssrc;
@@ -1296,8 +1036,9 @@ RTP_Session::SendReceiveStatus RTP_Session::OnReceiveControl(RTP_ControlFrame &
           sender.rtpTimestamp = sr.rtp_ts;
           sender.packetsSent = sr.psent;
           sender.octetsSent = sr.osent;
-          OnRxSenderReport(sender,
-                BuildReceiverReportArray(frame, sizeof(RTP_ControlFrame::SenderReport)));
+          // trace the report
+          ReceiverReportArray RRs = BuildReceiverReportArray(frame, sizeof(RTP_ControlFrame::SenderReport));
+          OnRxSenderReport(sender, RRs);
         }
         else {
           PTRACE(2, "RTP\tSenderReport packet truncated");
@@ -1305,7 +1046,7 @@ RTP_Session::SendReceiveStatus RTP_Session::OnReceiveControl(RTP_ControlFrame &
         break;
 
       case RTP_ControlFrame::e_ReceiverReport :
-        if (size >= 4)
+        if (size >= (frame.GetCount()*sizeof(RTP_ControlFrame::ReceiverReport)))
           OnRxReceiverReport(*(const PUInt32b *)payload,
                                       BuildReceiverReportArray(frame, sizeof(PUInt32b)));
         else {
@@ -1372,6 +1113,8 @@ RTP_Session::SendReceiveStatus RTP_Session::OnReceiveControl(RTP_ControlFrame &
 void RTP_Session::OnRxSenderReport(const SenderReport & PTRACE_PARAM(sender),
                                    const ReceiverReportArray & PTRACE_PARAM(reports))
 {
+   userData->OnRxSenderReport(sessionID,sender,reports);
+
 #if PTRACING
   PTRACE(3, "RTP\tOnRxSenderReport: " << sender);
   for (PINDEX i = 0; i < reports.GetSize(); i++)
@@ -1463,7 +1206,7 @@ void RTP_Session::SourceDescription::PrintOn(ostream & strm) const
 DWORD RTP_Session::GetPacketsTooLate() const
 {
   return
-#ifndef NO_H323_AUDIO_CODECS
+#ifdef H323_AUDIO_CODECS
     jitter != NULL ? jitter->GetPacketsTooLate() :
 #endif
   0;
@@ -1521,6 +1264,20 @@ void RTP_SessionManager::AddSession(RTP_Session * session)
   mutex.Signal();
 }
 
+void RTP_SessionManager::MoveSession(unsigned oldSessionID, unsigned newSessionID)
+{
+  PTRACE(2, "RTP\tMoving session " << oldSessionID << " to " << newSessionID);
+
+  mutex.Wait();
+
+  if (sessions.Contains(oldSessionID)) {
+      RTP_Session & session = sessions[oldSessionID];
+      sessions.SetAt(newSessionID, &session);
+      //sessions.RemoveAt(oldSessionID);
+  }
+
+  mutex.Signal();
+}
 
 void RTP_SessionManager::ReleaseSession(unsigned sessionID)
 {
@@ -1547,7 +1304,11 @@ RTP_Session * RTP_SessionManager::GetSession(unsigned sessionID) const
     return NULL;
 
   PTRACE(3, "RTP\tFound existing session " << sessionID);
+#if PTLIB_VER >= 2110
+  return const_cast<RTP_Session *>(&sessions[sessionID]);
+#else
   return &sessions[sessionID];
+#endif
 }
 
 
@@ -1646,7 +1407,7 @@ PBoolean RTP_UDP::ModifyQOS(RTP_QOS * rtpqos)
   if (rtpqos == NULL)
     return retval;
 
-#if P_HAS_QOS
+#if P_QOS
   if (dataSocket != NULL)
     retval &= dataSocket->ModifyQoSSpec(&(rtpqos->dataQoS));
   else if (controlSocket != NULL)
@@ -1659,17 +1420,17 @@ PBoolean RTP_UDP::ModifyQOS(RTP_QOS * rtpqos)
 
 void RTP_UDP::EnableGQoS(PBoolean success)
 {
-	enableGQOS = success;
+    enableGQOS = success;
 }
 
-#if P_HAS_QOS
+#if P_QOS
 PQoS & RTP_UDP::GetQOS()
 {
     if (dataSocket != NULL) 
        return dataSocket->GetQoSSpec();
     else if (controlSocket != NULL) 
        return controlSocket->GetQoSSpec();
-	else
+    else
        return *new PQoS(); 
 }
 #endif
@@ -1677,10 +1438,11 @@ PQoS & RTP_UDP::GetQOS()
 PBoolean RTP_UDP::Open(PIPSocket::Address _localAddress,
                    WORD portBase, WORD portMax,
                    BYTE tos,
-				   const H323Connection & connection,
 #ifdef P_STUN
+                   const H323Connection & connection,
                    PNatMethod * meth,
 #else
+                   const H323Connection &,
                    void *,
 #endif
                    RTP_QOS * rtpQos)
@@ -1699,7 +1461,7 @@ PBoolean RTP_UDP::Open(PIPSocket::Address _localAddress,
   PQoS * dataQos = NULL;
   PQoS * ctrlQos = NULL;
   if (rtpQos != NULL) {
-#if P_HAS_QOS
+#if P_QOS
     dataQos = &(rtpQos->dataQoS);
     ctrlQos = &(rtpQos->ctrlQoS);
 #endif
@@ -1707,15 +1469,22 @@ PBoolean RTP_UDP::Open(PIPSocket::Address _localAddress,
 
 #ifdef P_STUN
   if (meth != NULL) {
-	connection.OnSetRTPNat(GetSessionID(),*meth);
+    H323Connection::SessionInformation * info = 
+         connection.BuildSessionInformation(GetSessionID());
 
-    if (meth->CreateSocketPair(dataSocket, controlSocket)) {
+#if PTLIB_VER > 260
+    if (meth->CreateSocketPair(dataSocket, controlSocket, localAddress,(void *)info)) {
+#else
+    if (meth->CreateSocketPair(dataSocket, controlSocket, localAddress)) {
+#endif
       dataSocket->GetLocalAddress(localAddress, localDataPort);
       controlSocket->GetLocalAddress(localAddress, localControlPort);
-	  PTRACE(4, "RTP\tNAT Method " << meth->GetName() << " created NAT ports " << localDataPort << " " << localControlPort);
+      PTRACE(4, "RTP\tNAT Method " << meth->GetName() << " created NAT ports " << localDataPort << " " << localControlPort);
     }
     else
       PTRACE(1, "RTP\tNAT could not create socket pair!");
+
+    delete info;
   }
 #endif
 
@@ -1832,6 +1601,15 @@ PBoolean RTP_UDP::SetRemoteSocketInfo(PIPSocket::Address address, WORD port, PBo
   return remoteAddress != 0 && port != 0;
 }
 
+PBoolean RTP_UDP::PseudoRead(int & selectStatus)
+{
+#if PTLIB_VER >= 290
+    return (controlSocket->DoPseudoRead(selectStatus) || 
+            dataSocket->DoPseudoRead(selectStatus));
+#else
+    return false;
+#endif
+}
 
 PBoolean RTP_UDP::ReadData(RTP_DataFrame & frame, PBoolean loop)
 {
@@ -1839,7 +1617,10 @@ PBoolean RTP_UDP::ReadData(RTP_DataFrame & frame, PBoolean loop)
 #ifdef H323_RTP_AGGREGATE
     PTime start;
 #endif
-    int selectStatus = PSocket::Select(*dataSocket, *controlSocket, reportTimer);
+    int selectStatus = 0;
+
+    if (!PseudoRead(selectStatus))
+       selectStatus = PSocket::Select(*dataSocket, *controlSocket, reportTimer);
 #ifdef H323_RTP_AGGREGATE
     unsigned duration = (unsigned)(PTime() - start).GetMilliSeconds();
     if (duration > 50) {
@@ -1912,7 +1693,7 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(PUDPSocket & socket
 
       // If remote address never set from higher levels, then try and figure
       // it out from the first packet received.
-      if (!remoteAddress.IsValid()) {
+      if (remoteAddress.IsAny() || !remoteAddress.IsValid()) {
         remoteAddress = addr;
         PTRACE(4, "RTP\tSet remote address from first " << channelName
                << " PDU from " << addr << ':' << port);
@@ -1926,17 +1707,35 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(PUDPSocket & socket
           remoteControlPort = port;
       }
 
-      if (!remoteTransmitAddress.IsValid())
-        remoteTransmitAddress = addr;
+      if (remoteTransmitAddress.IsAny() || !remoteTransmitAddress.IsValid())
+             remoteTransmitAddress = addr;
+
       else if (remoteTransmitAddress != addr) {
-        PTRACE(1, "RTP_UDP\tSession " << sessionID << ", "
-               << channelName << " PDU from incorrect host, "
-                  " is " << addr << " should be " << remoteTransmitAddress);
-        return RTP_Session::e_IgnorePacket;
+#ifdef H323_H46024A
+          if (socket.IsAlternateAddress(addr,port)) {
+                remoteTransmitAddress = addr;
+                if (fromDataChannel) {
+                    remoteDataPort = port;
+                    // Fixes to makes sure sync,stats and jitter don't get screwed up 
+                    syncSourceIn = ((RTP_DataFrame &)frame).GetSyncSource();
+                    expectedSequenceNumber = ((RTP_DataFrame &)frame).GetSequenceNumber();
+#ifdef H323_AUDIO_CODECS
+                    if (jitter != NULL)  jitter->ResetFirstWrite();
+#endif
+                } else
+                    remoteControlPort = port;
+          } else
+#endif
+          {
+            PTRACE(1, "RTP_UDP\tSession " << sessionID << ", "
+                   << channelName << " PDU from incorrect host, "
+                      " is " << addr << " should be " << remoteTransmitAddress);
+            return RTP_Session::e_IgnorePacket;
+          }
       }
     }
 
-    if (remoteAddress.IsValid() && !appliedQOS) 
+    if (!remoteAddress.IsAny() && remoteAddress.IsValid() && !appliedQOS) 
       ApplyQOS(remoteAddress);
 
     return RTP_Session::e_ProcessPacket;
@@ -1949,6 +1748,11 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(PUDPSocket & socket
              << channelName << " port on remote not ready.");
       return RTP_Session::e_IgnorePacket;
 
+    case EMSGSIZE :
+      PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName
+             << " read packet too large");
+      return RTP_Session::e_IgnorePacket;
+
     case EAGAIN :
       // Shouldn't happen, but it does.
       return RTP_Session::e_IgnorePacket;
@@ -2010,7 +1814,7 @@ PBoolean RTP_UDP::WriteData(RTP_DataFrame & frame)
   }
 
   // Trying to send a PDU before we are set up!
-  if (!remoteAddress.IsValid() || remoteDataPort == 0)
+  if (remoteAddress.IsAny() || !remoteAddress.IsValid() || remoteDataPort == 0)
     return TRUE;
 
   switch (OnSendData(frame)) {
@@ -2047,7 +1851,7 @@ PBoolean RTP_UDP::WriteData(RTP_DataFrame & frame)
 PBoolean RTP_UDP::WriteControl(RTP_ControlFrame & frame)
 {
   // Trying to send a PDU before we are set up!
-  if (!remoteAddress.IsValid() || remoteControlPort == 0)
+  if (remoteAddress.IsAny() || !remoteAddress.IsValid() || remoteControlPort == 0)
     return true;
 
   while (!controlSocket->WriteTo(frame.GetPointer(), frame.GetCompoundSize(),
@@ -2063,7 +1867,7 @@ PBoolean RTP_UDP::WriteControl(RTP_ControlFrame & frame)
                << ", Write error on control port ("
                << controlSocket->GetErrorNumber(PChannel::LastWriteError) << "): "
                << controlSocket->GetErrorText(PChannel::LastWriteError));
-		return false;
+        return false;
     }
   }
 
