@@ -3,7 +3,7 @@
  *
  * H.323 transports handler
  *
- * Open H323 Library
+ * H323Plus Library
  *
  * Copyright (c) 1998-2000 Equivalence Pty. Ltd.
  *
@@ -26,557 +26,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: transports.cxx,v $
- * Revision 1.3  2008/09/27 06:18:57  shorne
- * BUG FIX: H323SignalPDU::Write to correctly handle NULL H323Connection Thx Nir Soffer
- *
- * Revision 1.2  2008/05/23 11:22:41  willamowius
- * switch BOOL to PBoolean to be able to compile with Ptlib 2.2.x
- *
- * Revision 1.1  2007/08/06 20:51:08  shorne
- * First commit of h323plus
- *
- * Revision 1.161.2.2  2007/07/22 14:19:20  shorne
- * Removed initialisation warning under windows
- *
- * Revision 1.161.2.1  2007/02/11 00:45:20  shorne
- * Added ability to disable NAT method on a call by call basis
- *
- * Revision 1.161  2006/08/23 06:43:15  csoutheren
- * Improved locking for calls that end very quickly after starting
- *
- * Revision 1.160  2006/07/18 07:10:47  csoutheren
- * Set default listener port rather than using zero
- *
- * Revision 1.159  2006/07/05 04:37:44  csoutheren
- * Applied 1488904 - SetPromiscuous(AcceptFromLastReceivedOnly) for T.38
- * Thanks to Vyacheslav Frolov
- *
- * Revision 1.158  2006/06/23 06:02:44  csoutheren
- * Added missing declarations for H.224 backport
- *
- * Revision 1.157  2006/06/23 03:20:21  shorne
- * Added unsolicited Information support
- *
- * Revision 1.156  2006/03/02 07:52:51  csoutheren
- * Ensure prompt close of channels when using aggregation
- * Ensure MonitorCallStatus called when using aggregation
- *
- * Revision 1.155  2006/01/30 06:49:07  csoutheren
- * Removed warning on Linux
- *
- * Revision 1.154  2006/01/30 01:11:27  csoutheren
- * Fixed compile warning on Linux
- *
- * Revision 1.153  2006/01/26 03:38:54  shorne
- * Added transport capability exchange
- *
- * Revision 1.152  2006/01/24 08:15:24  csoutheren
- * Implement outgoing H.225 aggregation, and H.245 aggregation (disabled by default)
- * More testing to do, but this looks good :)
- *
- * Revision 1.151  2006/01/23 05:58:26  csoutheren
- * Working outgoing H.225 aggregation (disabled by default)
- *
- * Revision 1.150  2006/01/20 00:32:24  csoutheren
- * First check-in of signalling aggregation code - incomplete and disabled by default
- *
- * Revision 1.149  2005/11/25 00:37:45  csoutheren
- * Applied patch #1274826 from Paul Nader
- * HandleFirstSignallingChannelPDU incomplete ReleaseComplete
- *
- * Revision 1.148  2005/11/22 03:38:46  shorne
- * Added ToS support to TCP Transports. thx Norbert Bartalsky (TOPCALL)
- *
- * Revision 1.147  2005/09/16 08:11:06  csoutheren
- * Added new error code
- * Added override for OnIncomingCall that can set return code
- *
- * Revision 1.146  2005/03/07 02:10:14  csoutheren
- * Fixed more problems with locahost detection
- *
- * Revision 1.145  2005/03/04 03:22:12  csoutheren
- * Fixed problem with multi-homed hosts sending localhost in some gatekeeper PDUs
- *
- * Revision 1.144  2005/01/16 20:39:44  csoutheren
- * Fixed problem with IPv6 INADDR_ANY
- *
- * Revision 1.143  2005/01/03 14:03:43  csoutheren
- * Added new configure options and ability to disable/enable modules
- *
- * Revision 1.142  2005/01/03 06:26:09  csoutheren
- * Added extensive support for disabling code modules at compile time
- *
- * Revision 1.141  2004/12/02 00:24:26  csoutheren
- * Fixed problem with gatekeeper getting requests from localhost
- *
- * Revision 1.140  2004/11/25 07:38:59  csoutheren
- * Ensured that external TCP address translation is performed when using STUN to handle UDP
- *
- * Revision 1.139  2004/08/24 08:11:26  csoutheren
- * Added initial support for receiving broadcasts on Linux
- *
- * Revision 1.138  2004/07/11 08:09:14  csoutheren
- * Patch for VxWorks thanks to Eize Slange
- *
- * Revision 1.137  2004/04/25 22:23:22  ykiryanov
- * Removed couple of ifdef BEOS. More functionality
- *
- * Revision 1.136  2004/04/03 08:28:07  csoutheren
- * Remove pseudo-RTTI and replaced with real RTTI
- *
- * Revision 1.135  2003/12/03 06:57:11  csoutheren
- * Protected against dwarf Q.931 PDUs
- *
- * Revision 1.134  2003/04/10 09:45:34  robertj
- * Added associated transport to new GetInterfaceAddresses() function so
- *   interfaces can be ordered according to active transport links. Improves
- *   interoperability.
- * Replaced old listener GetTransportPDU() with GetInterfaceAddresses()
- *   and H323SetTransportAddresses() functions.
- *
- * Revision 1.133  2003/04/10 00:58:54  craigs
- * Added functions to access to lists of interfaces
- *
- * Revision 1.132  2003/03/26 06:14:31  robertj
- * More IPv6 support (INADDR_ANY handling), thanks Sbastien Josset
- *
- * Revision 1.131  2003/03/21 05:24:54  robertj
- * Added setting of remote port in UDP transport constructor.
- *
- * Revision 1.130  2003/03/20 01:51:12  robertj
- * More abstraction of H.225 RAS and H.501 protocols transaction handling.
- *
- * Revision 1.129  2003/03/11 23:15:23  robertj
- * Fixed possible double delete of socket (crash) on garbage input.
- *
- * Revision 1.128  2003/02/06 04:31:02  robertj
- * Added more support for adding things to H323TransportAddressArrays
- *
- * Revision 1.127  2003/02/05 01:57:18  robertj
- * Fixed STUN usage on gatekeeper discovery.
- *
- * Revision 1.126  2003/02/04 07:06:42  robertj
- * Added STUN support.
- *
- * Revision 1.125  2003/01/23 02:36:32  robertj
- * Increased (and made configurable) timeout for H.245 channel TCP connection.
- *
- * Revision 1.124  2002/12/23 22:46:06  robertj
- * Changed gatekeeper discovery so an GRJ does not indicate "discovered".
- *
- * Revision 1.123  2002/11/21 06:40:00  robertj
- * Changed promiscuous mode to be three way. Fixes race condition in gkserver
- *   which can cause crashes or more PDUs to be sent to the wrong place.
- *
- * Revision 1.122  2002/11/12 03:14:18  robertj
- * Fixed gatekeeper discovery so does IP address translation correctly for
- *   hosts inside the firewall.
- *
- * Revision 1.121  2002/11/10 08:10:44  robertj
- * Moved constants for "well known" ports to better place (OPAL change).
- *
- * Revision 1.120  2002/11/05 00:31:48  robertj
- * Prevented a failure to start separate H.245 channel stopping the call until
- *   after a CONNECT is received and have no audio. At that point no H.245
- *   is a useless call and we disconnect.
- *
- * Revision 1.119  2002/11/01 03:38:18  robertj
- * More IPv6 fixes, thanks Sbastien Josset.
- *
- * Revision 1.118  2002/10/29 08:30:32  robertj
- * Fixed problem with simultaneous startH245 condition possibly shutting down
- *   the call under some circumstances.
- *
- * Revision 1.117  2002/10/16 06:28:20  robertj
- * More IPv6 support changes, especially in unambiguising v6 addresses colons
- *   from the port fields colon, thanks Sebastien Josset
- *
- * Revision 1.116  2002/10/08 23:34:30  robertj
- * Fixed ip v6 usage on H.245 pdu setting.
- *
- * Revision 1.115  2002/10/08 13:08:21  robertj
- * Changed for IPv6 support, thanks Sbastien Josset.
- *
- * Revision 1.114  2002/08/05 10:03:48  robertj
- * Cosmetic changes to normalise the usage of pragma interface/implementation.
- *
- * Revision 1.113  2002/08/05 05:17:41  robertj
- * Fairly major modifications to support different authentication credentials
- *   in ARQ to the logged in ones on RRQ. For both client and server.
- * Various other H.235 authentication bugs and anomalies fixed on the way.
- *
- * Revision 1.112  2002/07/22 09:40:19  robertj
- * Added ability to automatically convert string arrays, lists sorted lists
- *   directly to H323TransportAddressArray.
- *
- * Revision 1.111  2002/07/18 08:25:47  robertj
- * Fixed problem in decoding host when '+' was used without port in address.
- *
- * Revision 1.110  2002/07/10 01:23:33  robertj
- * Added extra debugging output
- *
- * Revision 1.109  2002/07/02 10:02:32  robertj
- * Added H323TransportAddress::GetIpAddress() so don't have to provide port
- *   when you don't need it as in GetIpAndPort(),.
- *
- * Revision 1.108  2002/06/28 03:34:29  robertj
- * Fixed issues with address translation on gatekeeper RAS channel.
- *
- * Revision 1.107  2002/06/24 07:35:23  robertj
- * Fixed ability to do gk discovery on localhost, thanks Artis Kugevics
- *
- * Revision 1.106  2002/06/12 03:52:27  robertj
- * Added function to compare two transport addresses in a more intelligent
- *   way that strict string comparison. Takes into account wildcarding.
- *
- * Revision 1.105  2002/05/28 06:38:08  robertj
- * Split UDP (for RAS) from RTP port bases.
- * Added current port variable so cycles around the port range specified which
- *   fixes some wierd problems on some platforms, thanks Federico Pinna
- *
- * Revision 1.104  2002/05/22 07:39:59  robertj
- * Fixed double increment of port number when making outgoing TCP connection.
- *
- * Revision 1.103  2002/04/18 00:18:58  robertj
- * Increased timeout for thread termination assert, on heavily loaded machines it can
- *   take more than one second to complete.
- *
- * Revision 1.102  2002/04/17 05:36:38  robertj
- * Fixed problems with using pre-bound inferface/port in gk discovery.
- *
- * Revision 1.101  2002/04/12 04:51:28  robertj
- * Fixed small possibility crashes if open and close transport at same time.
- *
- * Revision 1.100  2002/03/08 01:22:30  robertj
- * Fixed possible use of IsSuspended() on terminated thread causing assert.
- *
- * Revision 1.99  2002/03/05 04:49:41  robertj
- * Fixed leak of thread (and file handles) if get incoming connection aborted
- *   very early (before receiving a setup PDU), thanks Hans Bjurstrm
- *
- * Revision 1.98  2002/02/28 04:35:43  robertj
- * Added trace output of the socket handle number when have new connection.
- *
- * Revision 1.97  2002/02/28 00:57:03  craigs
- * Changed SetWriteTimeout to SetReadTimeout in connect, as Craig got it wrong!
- *
- * Revision 1.96  2002/02/25 10:55:33  robertj
- * Added ability to speficy dynamically allocated port in transport address.
- *
- * Revision 1.95  2002/02/14 03:36:14  craigs
- * Added default 10sec timeout on connect to IP addresses
- * This prevents indefinite hangs when connecting to IP addresses
- * that don't exist
- *
- * Revision 1.94  2002/02/05 23:29:09  robertj
- * Changed default for H.323 listener to reuse addresses.
- *
- * Revision 1.93  2002/02/01 01:48:18  robertj
- * Fixed ability to shut down a Listener, if it had never been started.
- *
- * Revision 1.92  2002/01/02 06:06:43  craigs
- * Made T.38 UDP socket obey rules
- *
- * Revision 1.91  2001/12/22 01:48:40  robertj
- * Added ability to use local and remote port from transport channel as well
- *   as explicit port in H.245 address PDU setting routine.
- * Added PrintOn() to listener and transport for tracing purposes.
- *
- * Revision 1.90  2001/12/15 07:12:22  robertj
- * Added optimisation so if discovering a static gk on same machine as ep is
- *   running on then uses that specific interface preventing multiple GRQs.
- *
- * Revision 1.89  2001/10/11 07:16:49  robertj
- * Removed port check for gk's that change sockets in mid-stream.
- *
- * Revision 1.88  2001/10/09 12:41:20  robertj
- * Set promiscuous flag back to FALSE after gatkeeper discovery.
- *
- * Revision 1.87  2001/09/10 03:06:29  robertj
- * Major change to fix problem with error codes being corrupted in a
- *   PChannel when have simultaneous reads and writes in threads.
- *
- * Revision 1.86  2001/08/10 11:03:52  robertj
- * Major changes to H.235 support in RAS to support server.
- *
- * Revision 1.85  2001/08/07 02:57:52  robertj
- * Improved tracing on closing transport.
- *
- * Revision 1.84  2001/08/06 03:08:57  robertj
- * Fission of h323.h to h323ep.h & h323con.h, h323.h now just includes files.
- *
- * Revision 1.83  2001/07/17 04:44:32  robertj
- * Partial implementation of T.120 and T.38 logical channels.
- *
- * Revision 1.82  2001/07/06 02:31:15  robertj
- * Made sure a release complete is sent if no connection is created.
- *
- * Revision 1.81  2001/07/04 09:02:07  robertj
- * Added more tracing
- *
- * Revision 1.80  2001/06/25 05:50:22  robertj
- * Improved error logging on TCP listener.
- *
- * Revision 1.79  2001/06/25 02:28:34  robertj
- * Allowed TCP listener socket to be opened in non-exclusive mode
- *   (ie SO_REUSEADDR) to avoid daemon restart problems.
- * Added trailing '+' on H323TransportAddress string to invoke above.
- *
- * Revision 1.78  2001/06/22 02:47:12  robertj
- * Took one too many lines out in previous fix!
- *
- * Revision 1.77  2001/06/22 02:40:27  robertj
- * Fixed discovery so uses new promiscuous mode.
- * Also used the RAS GRQ address of server isntead of UDP return address
- *   for address of gatekeeper for future packets.
- *
- * Revision 1.76  2001/06/22 01:54:47  robertj
- * Removed initialisation of localAddress to hosts IP address, does not
- *   work on multi-homed hosts.
- *
- * Revision 1.75  2001/06/22 00:14:46  robertj
- * Added ConnectTo() function to conencto specific address.
- * Added promiscuous mode for UDP channel.
- *
- * Revision 1.74  2001/06/14 23:18:06  robertj
- * Change to allow for CreateConnection() to return NULL to abort call.
- *
- * Revision 1.73  2001/06/14 04:23:32  robertj
- * Changed incoming call to pass setup pdu to endpoint so it can create
- *   different connection subclasses depending on the pdu eg its alias
- *
- * Revision 1.72  2001/06/06 00:29:54  robertj
- * Added trace for when doing TCP connect.
- *
- * Revision 1.71  2001/06/02 01:35:32  robertj
- * Added thread names.
- *
- * Revision 1.70  2001/05/31 07:16:52  craigs
- * Fixed remote address initialisation for incoming H245 channels
- *
- * Revision 1.69  2001/05/17 06:37:04  robertj
- * Added multicast gatekeeper discovery support.
- *
- * Revision 1.68  2001/04/13 07:44:51  robertj
- * Moved starting connection trace message to be on both Connect() and Accept()
- *
- * Revision 1.67  2001/04/10 01:21:02  robertj
- * Added some more error messages into trace log.
- *
- * Revision 1.66  2001/04/09 08:44:19  robertj
- * Added ability to get transport address for a listener.
- * Added '*' to indicate INADDR_ANY ip address.
- *
- * Revision 1.65  2001/03/06 05:03:00  robertj
- * Changed H.245 channel start failure so does not abort call if there were
- *   some fast started media streams opened. Just lose user indications.
- *
- * Revision 1.64  2001/03/05 04:28:50  robertj
- * Added net mask to interface info returned by GetInterfaceTable()
- *
- * Revision 1.63  2001/02/09 05:13:56  craigs
- * Added pragma implementation to (hopefully) reduce the executable image size
- * under Linux
- *
- * Revision 1.62  2001/02/08 22:29:39  robertj
- * Fixed failure to reset fill character in trace log when output interface list.
- *
- * Revision 1.61  2001/01/29 06:43:32  robertj
- * Added printing of entry of interface table.
- *
- * Revision 1.60  2001/01/23 05:06:52  robertj
- * Fixed bug when trying to clear call while waiting on H.245 channel connect, thanks Yura Aksyonov.
- * Fixed missing increment in H.245 listener when specified local port number range.
- *
- * Revision 1.59  2001/01/11 06:41:01  craigs
- * Fixed problem with outgoing sockets failing because of port in use
- *
- * Revision 1.58  2000/12/18 08:59:20  craigs
- * Added ability to set ports
- *
- * Revision 1.57  2000/11/06 06:19:19  robertj
- * Removed reverse DNS lookup for host name as this can cause HUGE delays in
- *    call setup making it look like a call failure.
- *
- * Revision 1.56  2000/10/20 06:18:49  robertj
- * Fixed very small race condition on creating new connectionon incoming call.
- * Fixed memory/socket leak if do TCP connect and don't send valid setup PDU.
- *
- * Revision 1.55  2000/10/04 12:21:07  robertj
- * Changed setting of callToken in H323Connection to be as early as possible.
- *
- * Revision 1.54  2000/10/04 05:59:26  robertj
- * Minor reorganisation of the H.245 secondary channel start up to make it simpler
- *    to override its behaviour.
- *
- * Revision 1.53  2000/09/25 12:59:34  robertj
- * Added StartListener() function that takes a H323TransportAddress to start
- *     listeners bound to specific interfaces.
- *
- * Revision 1.52  2000/08/31 13:14:10  robertj
- * Increased timeout on assert for stuck transport thread.
- *
- * Revision 1.51  2000/08/25 01:10:28  robertj
- * Added assert if various thrads ever fail to terminate.
- *
- * Revision 1.50  2000/08/22 09:01:06  robertj
- * Fixed small window in which control channel could do write through NULL pointer.
- *
- * Revision 1.49  2000/07/10 16:11:49  robertj
- * Fixed inclusion of interfaces with no IP address in GK registration.
- * Fixed possible crash when closing connection during H.245 TCP connect wait.
- *
- * Revision 1.48  2000/06/20 02:38:28  robertj
- * Changed H323TransportAddress to default to IP.
- *
- * Revision 1.47  2000/06/07 05:48:06  robertj
- * Added call forwarding.
- *
- * Revision 1.46  2000/05/23 02:38:15  robertj
- * Shortened the linger onTCP close to 3 seconds, should be adequate for lst end session transmit.
- *
- * Revision 1.45  2000/05/22 05:21:36  robertj
- * Fixed race condition where controlChannel variable could be used before set.
- *
- * Revision 1.44  2000/05/10 05:14:53  robertj
- * Fixed memory leak when doing GK discovery.
- *
- * Revision 1.43  2000/05/08 14:07:35  robertj
- * Improved the provision and detection of calling and caller numbers, aliases and hostnames.
- *
- * Revision 1.42  2000/05/05 04:37:50  robertj
- * Changed TCP transmit of PDU to not use Nagle algorithm delay, this gives
- *     a significant performance benefit for packet based stuff on stream based TCP.
- *
- * Revision 1.41  2000/05/02 04:32:28  robertj
- * Fixed copyright notice comment.
- *
- * Revision 1.40  2000/04/19 01:59:21  robertj
- * Improved some debugging messages.
- * Added code to restore transport state if gatekeeper discovery fails.
- *
- * Revision 1.39  2000/04/14 17:32:39  robertj
- * Changed transport to return some error number when getting malformed TPKT.
- *
- * Revision 1.38  2000/04/11 03:57:25  robertj
- * Fixed uninitialised variabel giving random port numbers for some PDU's (rare).
- *
- * Revision 1.37  2000/04/10 17:41:46  robertj
- * Changed transport address so does not do reverse DNS lookup, can really slow things down on some systems.
- *
- * Revision 1.36  2000/03/29 02:14:46  robertj
- * Changed TerminationReason to CallEndReason to use correct telephony nomenclature.
- * Added CallEndReason for capability exchange failure.
- *
- * Revision 1.35  2000/03/25 02:02:25  robertj
- * Added adjustable caller name on connection by connection basis.
- *
- * Revision 1.34  2000/02/09 00:00:53  robertj
- * Fixed TCP listener socket error message going to stderr, should go through PTRACE.
- *
- * Revision 1.33  2000/01/29 07:13:33  robertj
- * Fixed possible incorect port being used when GK RAS send address set to the
- *    IP address of the responding GK, thanks Stefan Ditscheid.
- *
- * Revision 1.32  2000/01/07 08:24:01  robertj
- * Added transport independent MakeCall requiring change to transport string format
- *
- * Revision 1.31  1999/12/09 21:46:42  robertj
- * Fixed detection of bound interface in gatekeeper discovery (can't use getsockname!)
- *
- * Revision 1.30  1999/12/09 20:25:41  robertj
- * Changed UDP transport to only do gatekeeper discovery on bound interface (if present)
- *
- * Revision 1.29  1999/11/22 00:56:35  robertj
- * Improved reason display for connection failure.
- *
- * Revision 1.28  1999/11/18 12:51:20  robertj
- * Fixed bug that termination connections incorrectly while awaiting answer.
- *
- * Revision 1.27  1999/11/06 05:37:45  robertj
- * Complete rewrite of termination of connection to avoid numerous race conditions.
- *
- * Revision 1.26  1999/10/28 11:17:38  robertj
- * Fixed bug causing delete of deleted object, thanks Benny Prijono
- *
- * Revision 1.25  1999/10/16 03:47:49  robertj
- * Fixed termination of gatekeeper RAS thread problem
- *
- * Revision 1.24  1999/10/14 12:04:11  robertj
- * Fixed ability to hang up call when still doing TCP connect.
- *
- * Revision 1.23  1999/10/13 04:32:41  robertj
- * Fixed missing port number from gatekeeper discovery.
- * Added removal of redundent transport listeners in H225 PDU.
- *
- * Revision 1.22  1999/10/10 14:12:06  robertj
- * Fixed failure to clean up connection if call is refused by user.
- *
- * Revision 1.21  1999/10/10 08:59:47  robertj
- * no message
- *
- * Revision 1.20  1999/09/27 01:37:23  robertj
- * BeOS port issues
- *
- * Revision 1.19  1999/09/17 07:26:17  robertj
- * Fixed attempt to broadcast to down interfaces or ones not bound to IP protocol.
- *
- * Revision 1.18  1999/09/14 08:19:37  robertj
- * Fixed timeout on retry of gatekeeper discover and added more tracing.
- *
- * Revision 1.17  1999/09/14 06:52:54  robertj
- * Added better support for multi-homed client hosts.
- *
- * Revision 1.16  1999/09/10 09:43:59  robertj
- * Removed attempt at determining local interface for gatekeeper, so still has problem on multi-homed hosts.
- *
- * Revision 1.15  1999/09/02 15:25:39  robertj
- * Old GNU C ompiler compatibility
- *
- * Revision 1.14  1999/08/31 12:34:19  robertj
- * Added gatekeeper support.
- *
- * Revision 1.13  1999/08/31 11:37:30  robertj
- * Fixed problem with apparently randomly losing signalling channel.
- *
- * Revision 1.12  1999/08/25 05:12:23  robertj
- * Changed MakeCall, so immediately spawns thread, no black on TCP connect.
- *
- * Revision 1.11  1999/07/23 02:37:53  robertj
- * Fixed problems with hang ups and crash closes of connections.
- *
- * Revision 1.10  1999/07/22 14:34:16  robertj
- * Fixed shut down problem, terminate packets being flushed on exit.
- *
- * Revision 1.9  1999/07/15 14:45:36  robertj
- * Added propagation of codec open error to shut down logical channel.
- * Fixed control channel start up bug introduced with tunnelling.
- *
- * Revision 1.8  1999/07/14 06:06:14  robertj
- * Fixed termination problems (race conditions) with deleting connection object.
- *
- * Revision 1.7  1999/07/09 06:09:52  robertj
- * Major implementation. An ENORMOUS amount of stuff added everywhere.
- *
- * Revision 1.6  1999/06/25 10:25:35  robertj
- * Added maintentance of callIdentifier variable in H.225 channel.
- *
- * Revision 1.5  1999/06/22 13:42:05  robertj
- * Added user question on listener version to accept incoming calls.
- *
- * Revision 1.4  1999/06/14 08:42:30  robertj
- * GNU C++ compatibility
- *
- * Revision 1.3  1999/06/13 12:41:14  robertj
- * Implement logical channel transmitter.
- * Fixed H245 connect on receiving call.
- *
- * Revision 1.2  1999/06/09 06:18:01  robertj
- * GCC compatibiltiy.
- *
- * Revision 1.1  1999/06/09 05:26:20  robertj
- * Major restructuring of classes.
+ * $Id: transports.cxx,v 1.12 2011/10/19 13:03:31 shorne Exp $
  *
  */
 
@@ -599,6 +49,9 @@
  #endif
 #endif
 
+// TCP KeepAlive 
+static int KeepAliveInterval = 19;
+
 class H225TransportThread : public PThread
 {
   PCLASSINFO(H225TransportThread, PThread)
@@ -606,10 +59,18 @@ class H225TransportThread : public PThread
   public:
     H225TransportThread(H323EndPoint & endpoint, H323Transport * transport);
 
+    ~H225TransportThread();
+
+    void ConnectionEstablished();
+
   protected:
     void Main();
 
     H323Transport * transport;
+
+	PDECLARE_NOTIFIER(PTimer, H225TransportThread, KeepAlive);
+	PTimer	m_keepAlive;								
+	PBoolean useKeepAlive;
 };
 
 
@@ -622,6 +83,8 @@ class H245TransportThread : public PThread
                         H323Connection & connection,
                         H323Transport & transport);
 
+    ~H245TransportThread();
+
   protected:
     void Main();
 
@@ -630,6 +93,9 @@ class H245TransportThread : public PThread
 #ifdef H323_SIGNAL_AGGREGATE
     PBoolean useAggregator;
 #endif
+
+	PDECLARE_NOTIFIER(PTimer, H245TransportThread, KeepAlive);
+	PTimer	m_keepAlive;			
 };
 
 
@@ -645,18 +111,54 @@ H225TransportThread::H225TransportThread(H323EndPoint & ep, H323Transport * t)
             "H225 Answer:%0x"),
     transport(t)
 {
+  useKeepAlive = ep.EnableH225KeepAlive();
   Resume();
 }
 
 
+H225TransportThread::~H225TransportThread()
+{
+    if (useKeepAlive)
+       m_keepAlive.Stop();
+}
+
+void H225TransportThread::ConnectionEstablished()
+{
+  if (useKeepAlive) {
+     PTRACE(3, "H225\tStarted KeepAlive");
+     m_keepAlive.SetNotifier(PCREATE_NOTIFIER(KeepAlive));
+     m_keepAlive.RunContinuous(KeepAliveInterval * 1000);
+  }
+}
+
+
 void H225TransportThread::Main()
 {
   PTRACE(3, "H225\tStarted incoming call thread");
 
-  if (!transport->HandleFirstSignallingChannelPDU())
+  if (!transport->HandleFirstSignallingChannelPDU(this))
     delete transport;
 }
 
+
+void H225TransportThread::KeepAlive(PTimer &, INT)
+{
+  // Send empty RFC1006 TPKT
+  int packetLength = 4;
+  PBYTEArray tpkt(packetLength);
+  memset(tpkt.GetPointer(),0 , packetLength);
+
+  tpkt[0] = 3;
+  tpkt[1] = 0;
+  tpkt[2] = (BYTE)(packetLength >> 8);
+  tpkt[3] = (BYTE)packetLength;
+
+  PTRACE(6, "H225\tSending KeepAlive TPKT packet");
+
+  if (transport)
+     transport->Write((const BYTE *)tpkt, packetLength);
+}
+
 /////////////////////////////////////////////////////////////////////////////
 
 H245TransportThread::H245TransportThread(H323EndPoint & endpoint,
@@ -675,11 +177,21 @@ H245TransportThread::H245TransportThread(H323EndPoint & endpoint,
 #endif
   {
     transport.AttachThread(this);
+    if (endpoint.EnableH245KeepAlive()) {
+      m_keepAlive.SetNotifier(PCREATE_NOTIFIER(KeepAlive));
+      m_keepAlive.RunContinuous(KeepAliveInterval * 1000);
+    }
   }
   Resume();
 }
 
 
+H245TransportThread::~H245TransportThread()
+{
+  m_keepAlive.Stop();
+}
+
+
 void H245TransportThread::Main()
 {
   PTRACE(3, "H245\tStarted thread");
@@ -700,6 +212,22 @@ void H245TransportThread::Main()
 }
 
 
+void H245TransportThread::KeepAlive(PTimer &, INT)
+{
+  // Send empty RFC1006 TPKT
+  int packetLength = 4;
+  PBYTEArray tpkt(packetLength);
+  tpkt[0] = 3;
+  tpkt[1] = 0;
+  tpkt[2] = 0;
+  tpkt[3] = (BYTE)packetLength;
+
+  PTRACE(6, "H245\tSending KeepAlive TPKT packet");
+
+  transport.Write((const BYTE *)tpkt, packetLength);
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 static const char IpPrefix[] = "ip$";
@@ -724,7 +252,7 @@ static PString BuildIP(const PIPSocket::Address & ip, unsigned port)
 
   str << IpPrefix;
 
-  if (!ip.IsValid())
+  if (ip.IsAny() || !ip.IsValid())
     str << '*';
   else
 #if P_HAS_IPV6
@@ -748,6 +276,7 @@ H323TransportAddress::H323TransportAddress(const H225_TransportAddress & transpo
     {
       const H225_TransportAddress_ipAddress & ip = transport;
       *this = BuildIP(PIPSocket::Address(ip.m_ip.GetSize(), ip.m_ip.GetValue()), ip.m_port);
+      m_version = 4;
       break;
     }
 #if P_HAS_IPV6
@@ -755,6 +284,7 @@ H323TransportAddress::H323TransportAddress(const H225_TransportAddress & transpo
     {
       const H225_TransportAddress_ip6Address & ip = transport;
       *this = BuildIP(PIPSocket::Address(ip.m_ip.GetSize(), ip.m_ip.GetValue()), ip.m_port);
+      m_version = 6;
       break;
     }
 #endif
@@ -773,6 +303,7 @@ H323TransportAddress::H323TransportAddress(const H245_TransportAddress & transpo
         {
           const H245_UnicastAddress_iPAddress & ip = unicast;
           *this = BuildIP(PIPSocket::Address(ip.m_network.GetSize(), ip.m_network.GetValue()), ip.m_tsapIdentifier);
+          m_version = 4;
           break;
         }
 #if P_HAS_IPV6
@@ -780,6 +311,7 @@ H323TransportAddress::H323TransportAddress(const H245_TransportAddress & transpo
         {
           const H245_UnicastAddress_iP6Address & ip = unicast;
           *this = BuildIP(PIPSocket::Address(ip.m_network.GetSize(), ip.m_network.GetValue()), ip.m_tsapIdentifier);
+          m_version = 6;
           break;
         }
 #endif
@@ -792,6 +324,11 @@ H323TransportAddress::H323TransportAddress(const H245_TransportAddress & transpo
 
 H323TransportAddress::H323TransportAddress(const PIPSocket::Address & ip, WORD port)
 {
+#if P_HAS_IPV6
+   m_version = ip.GetVersion();
+#else
+   m_version = 4;
+#endif
   *this = BuildIP(ip, port);
 }
 
@@ -801,6 +338,11 @@ void H323TransportAddress::Validate()
   if (IsEmpty())
     return;
 
+  if (Find(']') != P_MAX_INDEX)
+     m_version = 6;
+  else
+     m_version = 4;
+
   if (Find('$') == P_MAX_INDEX) {
     Splice(IpPrefix, 0, 0);
     return;
@@ -955,12 +497,33 @@ PBoolean H323TransportAddress::GetIpAndPort(PIPSocket::Address & ip,
   }
 
   if (PIPSocket::GetHostAddress(host, ip))
-    return TRUE;
+     return TRUE;
+
+#if P_HAS_IPV6
+  // This is really horrible
+  // You first attempt to get an IPv6 (default) record then if the fails
+  // Set the defaultIPAddress family to v4, Clear the cache then resolve the address via IPv4.
+  // This really needs to be cleaned up in PTLIB  - SH
+  if (PIPSocket::GetDefaultIpAddressFamily() == AF_INET6) {
+      PTRACE(3, "H323\tCould not resolve IPv6 Address for : \"" << host << '"' << " Trying IPv4:");
+      PIPSocket::SetDefaultIpAddressFamilyV4(); 
+      PIPSocket::ClearNameCache();  // clear the IPv6 record
+      bool success = PIPSocket::GetHostAddress(host, ip);
+      PIPSocket::SetDefaultIpAddressFamilyV6();
+      if (success)
+          return TRUE;
+  }
+#endif
 
   PTRACE(1, "H323\tCould not find host : \"" << host << '"');
   return FALSE;
 }
 
+unsigned H323TransportAddress::GetIpVersion() const
+{
+    return m_version;
+}
+
 
 PString H323TransportAddress::GetHostName() const
 {
@@ -1021,7 +584,7 @@ H323Transport * H323TransportAddress::CreateTransport(H323EndPoint & endpoint) c
    */
 
   if (strncmp(theArray, IpPrefix, 3) == 0)
-    return new H323TransportTCP(endpoint);
+      return new H323TransportTCP(endpoint,PIPSocket::Address::GetAny(m_version));
 
   return NULL;
 }
@@ -1254,7 +817,7 @@ PBoolean H323Transport::HandleSignallingSocket(H323SignalPDU & pdu)
   return FALSE;
 }
 
-PBoolean H323Transport::HandleFirstSignallingChannelPDU()
+PBoolean H323Transport::HandleFirstSignallingChannelPDU(PThread * thread)
 {
   PTRACE(3, "H225\tAwaiting first PDU");
   SetReadTimeout(15000); // Await 15 seconds after connect for first byte
@@ -1315,7 +878,8 @@ PBoolean H323Transport::HandleFirstSignallingChannelPDU()
     // which is in turn attached to the connection so everything from gets cleaned up by the 
     // H323 cleaner thread from now on. So thread must not auto delete and the "transport" 
     // variable is not deleted either
-    PThread * thread = PThread::Current();
+    PAssert(PIsDescendant(thread, H225TransportThread), PInvalidCast);
+    ((H225TransportThread *)thread)->ConnectionEstablished();
     AttachThread(thread);
     thread->SetNoAutoDelete();
 
@@ -1475,7 +1039,8 @@ H323Transport * H323ListenerTCP::Accept(const PTimeInterval & timeout)
   PTRACE(4, "TCP\tWaiting on socket accept on " << GetTransportAddress());
   PTCPSocket * socket = new PTCPSocket;
   if (socket->Accept(listener)) {
-    H323TransportTCP * transport = new H323TransportTCP(endpoint);
+    unsigned m_version = GetTransportAddress().GetIpVersion();
+    H323TransportTCP * transport = new H323TransportTCP(endpoint, PIPSocket::Address::GetAny(m_version));
     if (transport->Open(socket))
       return transport;
 
@@ -1971,8 +1536,11 @@ H323TransportUDP::H323TransportUDP(H323EndPoint & ep,
 
   PTRACE(3, "H323UDP\tBinding to interface: " << binding << ':' << localPort);
 
+#if PTLIB_VER >= 2110
+  canGetInterface = binding.IsAny();
+#else
   canGetInterface = (binding.IsAny()) && udp->SetCaptureReceiveToAddress();
-
+#endif
 }
 
 
@@ -1998,7 +1566,11 @@ PBoolean H323TransportUDP::Connect()
 #ifdef P_STUN
   PSTUNClient * stun = endpoint.GetSTUN(remoteAddress);
   if (stun != NULL) {
+#if PTLIB_VER >= 2110
+    if (stun->CreateSocket(PNatMethod::eComponent_Unknown,socket)) {
+#else
     if (stun->CreateSocket(socket)) {
+#endif
       Open(socket);
       socket->GetLocalAddress(localAddress, localPort);
       PTRACE(4, "H323UDP\tSTUN created socket: " << localAddress << ':' << localPort);
@@ -2046,8 +1618,16 @@ PBoolean H323TransportUDP::ReadPDU(PBYTEArray & pdu)
 
     PUDPSocket * socket = (PUDPSocket *)GetReadChannel();
 
+#if PTLIB_VER >= 2110
+    if (canGetInterface) {
+      WORD notused;
+      // TODO: verify that this actually does the same as the pre 2.11.x version
+      socket->GetLastReceiveAddress(lastReceivedInterface, notused);
+    }
+#else
     if (canGetInterface)
       lastReceivedInterface = socket->GetLastReceiveToAddress();
+#endif
 
     PIPSocket::Address address;
     WORD port;
@@ -2102,56 +1682,93 @@ PBoolean H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
 
   PTRACE(3, "H225\tStarted gatekeeper discovery of \"" << address << '"');
 
-  PIPSocket::Address destAddr = INADDR_BROADCAST;
+  PIPSocket::Address destAddr;
+#if P_HAS_IPV6
+  if (address.GetIpVersion() == 6)
+      destAddr = PIPSocket::Address::GetBroadcast(6);
+  else
+#endif
+      destAddr = INADDR_BROADCAST;
+
+    // Skip over the H323Transport::Close to make sure PUDPSocket is deleted.
+  PIndirectChannel::Close();
+
   WORD destPort = H225_RAS::DefaultRasUdpPort;
   if (!address) {
     if (!address.GetIpAndPort(destAddr, destPort, "udp")) {
       PTRACE(2, "RAS\tError decoding address");
       return FALSE;
     }
+    remoteAddress = destAddr;
+    remotePort = destPort;
+  } else {
+    remoteAddress = 0;
+    remotePort = 0;
   }
 
-  // Skip over the H323Transport::Close to make sure PUDPSocket is deleted.
-  PIndirectChannel::Close();
-
-  remoteAddress = 0;
-  remotePort = 0;
 
   // Remember the original info for pre-bound socket
   PIPSocket::Address originalLocalAddress = localAddress;
   WORD originalLocalPort = 0;
 
+#if P_HAS_IPV6
+  // Again horrible code should be able to get interface listing for a given protocol - SH
+  PBoolean ipv6IPv4Discover = false;
+  if (address.GetIpVersion() == 4 && PIPSocket::GetDefaultIpAddressFamily() == AF_INET6) {
+      PIPSocket::SetDefaultIpAddressFamilyV4();
+      ipv6IPv4Discover = true;
+  }
+#endif
+
   // Get the interfaces to try
   PIPSocket::InterfaceTable interfaces;
+  PIPSocket::InterfaceTable InterfaceList;
 
   // See if prebound to interface, only use that if so
   if (destAddr.IsLoopback()) {
     PTRACE(3, "RAS\tGatekeeper discovery on loopback interface");
     localAddress = destAddr;
   }
-  else if (!localAddress.IsAny()) {
-    PTRACE(3, "RAS\tGatekeeper discovery on pre-bound interface: "
-              << localAddress << ':' << localPort);
-    originalLocalPort = localPort;
-  }
-  else if (!PIPSocket::GetInterfaceTable(interfaces)) {
+  else if (!PIPSocket::GetInterfaceTable(InterfaceList)) {
     PTRACE(1, "RAS\tNo interfaces on system!");
   }
-  else {
-    PTRACE(4, "RAS\tSearching interfaces:\n" << setfill('\n') << interfaces << setfill(' '));
-    // Check for if destination machine is local machine, if so only use that interface
-    for (i = 0; i < interfaces.GetSize(); i++) {
-      if (interfaces[i].GetAddress() == destAddr) {
-        PTRACE(3, "RAS\tGatekeeper discovery on local interface: " << destAddr);
-        localAddress = destAddr;
-        interfaces.RemoveAll();
+  else if (!localAddress.IsAny() && !localAddress.IsLoopback()) {
+    PTRACE(3, "RAS\tGatekeeper discovery on pre-bound interface: "
+              << localAddress.AsString(true) << ':' << localPort);
+    originalLocalPort = localPort;
+    for (i = 0; i < InterfaceList.GetSize(); i++) {
+      if (InterfaceList[i].GetAddress() == localAddress) {
+        PTRACE(3, "RAS\tGatekeeper local interface set: " << localAddress.AsString(true));
+        interfaces.Append(new PIPSocket::InterfaceEntry(InterfaceList[i].GetName(), 
+                                                        InterfaceList[i].GetAddress(), 
+                                                        InterfaceList[i].GetNetMask(), 
+                                                        InterfaceList[i].GetMACAddress()));
       }
     }
+    InterfaceList.RemoveAll();
+  } else {
+    for (i = 0; i < InterfaceList.GetSize(); i++) {
+      if (!InterfaceList[i].GetAddress().IsLoopback() &&
+           InterfaceList[i].GetAddress().GetVersion() == destAddr.GetVersion()) {
+            interfaces.Append(new PIPSocket::InterfaceEntry(InterfaceList[i].GetName(), 
+                                                            InterfaceList[i].GetAddress(), 
+                                                            InterfaceList[i].GetNetMask(), 
+                                                            InterfaceList[i].GetMACAddress()));
+      }
+    }
+    InterfaceList.RemoveAll();
   }
 
+#if P_HAS_IPV6
+  if (ipv6IPv4Discover)
+      PIPSocket::SetDefaultIpAddressFamilyV6();
+#endif
+
   if (interfaces.IsEmpty())
     interfaces.Append(new PIPSocket::InterfaceEntry("", localAddress, PIPSocket::Address(0xffffffff), ""));
 
+  
+
 #ifdef P_STUN
   PSTUNClient * stun = endpoint.GetSTUN(remoteAddress);
 #endif
@@ -2162,6 +1779,11 @@ PBoolean H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
 
   for (i = 0; i < interfaces.GetSize(); i++) {
     localAddress = interfaces[i].GetAddress();
+
+	// don't try to use IPv4 interface to reach IPv6 gatekeeper or IPv6 interface to reach IPv4 gatekeeper
+    if (localAddress.GetVersion() != destAddr.GetVersion())
+      continue;
+
     if (localAddress == 0 || (destAddr != localAddress && localAddress.IsLoopback()))
       continue;
 
@@ -2181,7 +1803,11 @@ PBoolean H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
 
 #ifdef P_STUN
       // Not explicitly multicast
+#if PTLIB_VER >= 2110
+      if (stun != NULL && stun->CreateSocket(PNatMethod::eComponent_Unknown,socket)) {
+#else
       if (stun != NULL && stun->CreateSocket(socket)) {
+#endif
         socket->GetLocalAddress(localAddress, localPort);
         PTRACE(4, "H323UDP\tSTUN created socket: " << localAddress << ':' << localPort);
       }
@@ -2231,8 +1857,10 @@ PBoolean H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
       socket = new PUDPSocket;
       sockets.Append(socket);
 
-      if (!ListenUDP(*socket, endpoint, localAddress, 0))
+	  if (!ListenUDP(*socket, endpoint, localAddress, 0)) {
+	    writeChannel = NULL;
         return FALSE;
+	  }
 
       localPort = socket->GetPort();
 
